@@ -92,6 +92,7 @@ namespace eval ::annotate {
 
       proc svn_annotate_color {w now logline ln} {
         global cvscfg
+        global cvsglb
         variable revcolors
         variable agecolors
         variable revlist
@@ -197,22 +198,25 @@ namespace eval ::annotate {
 
       # Make the window 
       toplevel $w
-      text $w.text -setgrid yes -relief sunken -border 2 \
-        -height 40 -width 122 -yscroll "$w.scroll set"
+      text $w.text -setgrid yes -exportselection 1 \
+        -relief sunken -border 2 -height 40 -width 122 \
+        -yscroll "$w.scroll set"
       scrollbar $w.scroll -relief sunken -command "$w.text yview"
 
 
       frame $w.bottom
-      button $w.bottom.close -text "Close" -command "destroy $w; exit_cleanup 0"
+      button $w.bottom.close -text "Close" -highlightbackground $cvsglb(bg) \
+        -command "destroy $w; exit_cleanup 0"
       label $w.bottom.days -text "Revs per Color" -width 20 -anchor e
       checkbutton $w.bottom.linum -text "Show Line Numbers" \
         -variable cvscfg(blame_linenums) \
         -onvalue 1 -offvalue 0
       entry $w.bottom.dayentry -width 3 \
         -textvariable [namespace current]::revspercolor
-      button $w.bottom.redo -text "Redo Colors"
+      button $w.bottom.redo -text "Redo Colors" -highlightbackground $cvsglb(bg)
 
-      button $w.bottom.srchbtn -text Search -command "search_textwidget $w.text"
+      button $w.bottom.srchbtn -text Search -highlightbackground $cvsglb(bg) \
+        -command "search_textwidget $w.text"
       entry $w.bottom.entry -width 20 -textvariable cvsglb(searchstr)
       bind $w.bottom.entry <Return> "search_textwidget $w.text"
 
