@@ -57,6 +57,7 @@ namespace eval ::logcanvas {
       set sel_tag(A) {}
       set sel_tag(B) {}
       variable sel_rev
+      variable revnum_current
       set sel_rev(A) {}
       set sel_rev(B) {}
       variable logcanvas ".logcanvas$my_idx"
@@ -142,10 +143,11 @@ namespace eval ::logcanvas {
         variable logcanvas
         variable sys
         variable loc
+        variable revnum_current
 
         switch -- $sys {
           "SVN" {
-            # Find out if it's a directory, if we can
+            # Find out current rev and if it's a directory, if we can
             set kind ""
             set info_cmd [exec::new "svn info \"[file tail $fname]\""]
             set info_lines [split [$info_cmd\::output] "\n"]
@@ -153,8 +155,14 @@ namespace eval ::logcanvas {
               if {[string match "Node Kind:*" $infoline]} {
                 gen_log:log D "$infoline"
                 set kind [lindex $infoline end]
+              } elseif {[string match "Last Changed Rev:*" $infoline]} {
+                gen_log:log D "$infoline"
+                set revnum_current [lindex $infoline end]
               }
             }
+            if {! [info exists revnum_current]} {
+                gen_log:log E "Warning: couldn't find current revision number!"
+            }
             $logcanvas.up.bmodbrowse configure -command {modbrowse_run svn} \
               -image Modules_svn
             $logcanvas.up.lfname configure -text "SVN Path"
@@ -163,9 +171,12 @@ namespace eval ::logcanvas {
             $logcanvas.up.rfname configure -state readonly -bg $cvsglb(textbg)
             $logcanvas.log configure \
                 -command [namespace code {
-                    # FIXME: we get the rev and dont use it
                     set rev [$logcanvas.up.revA_rvers cget -text] 
-                    svn_log $filename
+                    if {$rev =="" || [file isdirectory $filename]} {
+                      svn_log $filename
+                    } else {
+                      svn_log $revpath($rev) $filename
+                    }
                  }]
             if {$kind == "directory"} {
               $logcanvas.diff configure -state disabled
@@ -173,12 +184,14 @@ namespace eval ::logcanvas {
               $logcanvas.view configure \
                  -command [namespace code {
                     set rev [$logcanvas.up.revA_rvers cget -text] 
+                    if {$rev ==""} { set rev "r$revnum_current" }
                     svn_fileview $rev $revpath($rev) directory
                  }]
             } else {
               $logcanvas.view configure \
                  -command [namespace code {
                     set rev [$logcanvas.up.revA_rvers cget -text] 
+                    if {$rev ==""} { set rev "r$revnum_current" }
                     svn_fileview $rev $revpath($rev) file
                  }]
               $logcanvas.diff configure \
@@ -187,8 +200,11 @@ namespace eval ::logcanvas {
                    set revB [$logcanvas.up.revB_rvers cget -text]
                    set A [string trimleft $revA {r}]
                    set B [string trimleft $revB {r}]
+                   # Let's be generous and let either A or B be selected
                    if {$revB == ""} {
-                     comparediff_files $logcanvas "$revpath($revA)@$A" [file tail $revpath($revA)]
+                     comparediff_r "$revpath($revA)@$A" "" $logcanvas $filename
+                   } elseif {$revA == ""} {
+                     comparediff_r "" "$revpath($revB)@$B" $logcanvas $filename
                    } else {
                      comparediff_files $logcanvas "$revpath($revA)@$A" "$revpath($revB)@$B"
                    }
@@ -196,18 +212,19 @@ namespace eval ::logcanvas {
               $logcanvas.annotate configure \
                  -command [namespace code {
                    set rev [$logcanvas.up.revA_rvers cget -text]
-                   set R [string trimleft $rev {r}]
-                   if {$R == ""} {
-                     svn_annotate_r $R $filename
+                   if {$rev == ""} {
+                     svn_annotate_r "" $filename
                    } else {
-                     svn_annotate_r $R $revpath($rev)
+                     svn_annotate_r [string trimleft $rev {r}] $revpath($rev)
                    }
                  }]
             }
             $logcanvas.delta configure \
               -command [namespace code {
-                 variable sys
+                 set currentrevpath "$revpath(r$revnum_current)@$revnum_current"
                  set fromrev [$logcanvas.up.revA_rvers cget -text]
+                 if {$fromrev == ""} {cvsfail "Please select a revision!" $logcanvas; return}
+                 set fromrevpath "$revpath($fromrev)@[string trimleft $fromrev {r}]"
                  set sincerev [$logcanvas.up.revB_rvers cget -text]
                  set fromtag ""
                  if {[info exists revbtags($sincerev)]} {
@@ -223,10 +240,13 @@ namespace eval ::logcanvas {
                      }
                    }
                  }
-                 merge_dialog $sys \
-                   $fromrev $sincerev $fromtag \
-                   [list $revpath($sincerev)]
-                 }]
+                 if {$sincerev == ""} {
+                   svn_merge $logcanvas $fromrevpath "" $currentrevpath $fromtag $filename
+                 } else {
+                   set sincerevpath "$revpath($sincerev)@[string trimleft $sincerev {r}]"
+                   svn_merge $logcanvas $fromrevpath $sincerev $sincerevpath $fromtag $filename
+                 }
+               }]
           }
          "CVS" {
            $logcanvas.up.bmodbrowse configure -command {modbrowse_run cvs} \
@@ -275,24 +295,17 @@ namespace eval ::logcanvas {
                }]
              $logcanvas.delta configure \
                -command [namespace code {
-                 variable sys
                  set fromrev [$logcanvas.up.revA_rvers cget -text]
                  set sincerev [$logcanvas.up.revB_rvers cget -text]
                  set fromtag ""
-                 if {[info exists revbtags($sincerev)]} {
-                   set fromtag [lindex $revbtags($sincerev) 0]
+                 set fromrev_root [join [lrange [split $fromrev {.}] 0 end-1] {.}]
+                 if {[info exists revbtags($fromrev_root)]} {
+                   set fromtag [lindex $revbtags($fromrev_root) 0]
+                 } else {
+                   # Just a rev number will do
+                   set fromtag $fromrev_root
                  }
-                 if {$fromtag == ""} {
-                   foreach brev [array names branchrevs] {
-                     set b $branchrevs($brev)
-                     if {$b == $fromrev} {
-                       set fromtag $revbtags($brev)
-                     }
-                   }
-                 }
-                 merge_dialog $sys \
-                   $fromrev $sincerev $fromtag \
-                   [list $filename]
+                 cvs_merge $logcanvas $fromrev $sincerev $fromtag [list $filename]
                 }]
             }
          }
@@ -302,8 +315,7 @@ namespace eval ::logcanvas {
            $logcanvas.up.rfname configure -state readonly -bg $cvsglb(textbg)
            $logcanvas.view configure -state disabled
            $logcanvas.annotate configure -state disabled
-           $logcanvas.log configure -command [namespace code \
-                                               {rcs_log $filename}]
+           $logcanvas.log configure -command [namespace code {rcs_log $filename}]
            $logcanvas.delta configure -state disabled
           }
         }
@@ -1166,7 +1178,7 @@ namespace eval ::logcanvas {
             }
           }
           # If there's no trunk, find the beginning of a branch
-          if {! [info exists trunkrev]} {
+          if {! [info exists trunkrev] || $trunkrev == ""} {
             set min 999999
             foreach a [array names revbtags] {
               if {$a == "" } {continue}
@@ -1183,7 +1195,7 @@ namespace eval ::logcanvas {
           }
 
           # Start drawing, beginning with the trunk or the lowest branch
-          if {[info exists trunkrev]} {
+          if {[info exists trunkrev] && $trunkrev != ""} {
             gen_log:log D "Drawing trunkrev $trunkrev"
             foreach {lx y2 lbw rh lly} [DrawBranch 0 0 {} $trunkrev] {
               lappend bxys $lx $lbw $rh $lly
@@ -1279,7 +1291,6 @@ namespace eval ::logcanvas {
       proc SaveOptions {} {
         global logcfg
         variable opt
-        variable sys
         variable loc
 
         # Save the options to the global set
@@ -1312,7 +1323,6 @@ namespace eval ::logcanvas {
         -command [namespace code {$logcanvas.close invoke}]
       $logcanvas.menubar.file add command -label "Exit" -underline 1 \
         -command { exit_cleanup 1 }
-      set selcolor [option get $logcanvas selectColor selectColor]
       $logcanvas.menubar add cascade -label "View"\
          -menu $logcanvas.menubar.view -underline 0
       menu $logcanvas.menubar.view -tearoff 0
@@ -1320,13 +1330,10 @@ namespace eval ::logcanvas {
         -menu $logcanvas.menubar.view.update
       menu $logcanvas.menubar.view.update
       $logcanvas.menubar.view.update add radiobutton -label "Every Revision" \
-        -selectcolor $selcolor \
         -variable [namespace current]::opt(update_drawing) -value 0
       $logcanvas.menubar.view.update add radiobutton -label "Every Branch" \
-        -selectcolor $selcolor \
         -variable [namespace current]::opt(update_drawing) -value 1
       $logcanvas.menubar.view.update add radiobutton -label "When Finished" \
-        -selectcolor $selcolor \
         -variable [namespace current]::opt(update_drawing) -value 2
       $logcanvas.menubar.view add separator
       $logcanvas.menubar.view add cascade -label "Tree Layout" \
@@ -1336,19 +1343,16 @@ namespace eval ::logcanvas {
         "Show empty branches" \
         -variable [namespace current]::opt(show_empty_branches) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.tree add checkbutton -label \
         "Show intermediate revisions" \
         -variable [namespace current]::opt(show_inter_revs) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.tree add checkbutton -label \
         "Show merges" \
         -variable [namespace current]::opt(show_merges) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view add cascade -label "Branch Layout" \
         -menu $logcanvas.menubar.view.branch
@@ -1367,12 +1371,10 @@ namespace eval ::logcanvas {
       $logcanvas.menubar.view.branch add checkbutton -label "Show revision" \
         -variable [namespace current]::opt(show_root_rev) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.branch add checkbutton -label "Show label" \
         -variable [namespace current]::opt(show_root_tags) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view add cascade -label "Revision Layout" \
         -menu $logcanvas.menubar.view.rev
@@ -1399,27 +1401,22 @@ namespace eval ::logcanvas {
       $logcanvas.menubar.view.rev add checkbutton -label "Show tags" \
         -variable [namespace current]::opt(show_tags) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.rev add checkbutton -label "Show revision" \
         -variable [namespace current]::opt(show_box_rev) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.rev add checkbutton -label "Show author" \
         -variable [namespace current]::opt(show_box_revwho) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.rev add checkbutton -label "Show date" \
         -variable [namespace current]::opt(show_box_revdate) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view.rev add checkbutton -label "Show time" \
         -variable [namespace current]::opt(show_box_revtime) \
         -onvalue 1 -offvalue 0 \
-        -selectcolor $selcolor \
         -command [namespace code { DrawTree }]
       $logcanvas.menubar.view add separator
       $logcanvas.menubar.view add cascade -label "Size" \
@@ -1428,7 +1425,6 @@ namespace eval ::logcanvas {
       foreach {label factor} $logcfg(scaling_options) {
         $logcanvas.menubar.view.size add radiobutton -label $label \
           -variable [namespace current]::opt(scale) -value $factor \
-          -selectcolor $selcolor \
           -command [namespace code { DrawTree }]
       }
       $logcanvas.menubar.view add separator
@@ -1449,8 +1445,10 @@ namespace eval ::logcanvas {
       entry $logcanvas.up.rfname -font $textfont -relief groove \
         -readonlybackground $cvsglb(readonlybg)
         
-      button $logcanvas.up.bmodbrowse -image Modules -command modbrowse_run
-      button $logcanvas.up.bworkdir -image Workdir -command { workdir_setup }
+      button $logcanvas.up.bmodbrowse -image Modules -highlightbackground $cvsglb(bg) \
+        -command modbrowse_run
+      button $logcanvas.up.bworkdir -image Workdir -highlightbackground $cvsglb(bg) \
+        -command { workdir_setup }
       pack $logcanvas.up -side top -fill x
       foreach fm {A B} {
         label $logcanvas.up.rev${fm}_lvers -text "Revision $fm"
@@ -1519,20 +1517,20 @@ namespace eval ::logcanvas {
       #
       frame $logcanvas.down.btnfm
       frame $logcanvas.down.closefm -relief groove -bd 2
-      button $logcanvas.refresh -image Refresh \
+      button $logcanvas.refresh -image Refresh -highlightbackground $cvsglb(bg) \
         -command [namespace code {
                  $scope\::reloadLog
                }]
-      button $logcanvas.view -image Fileview
-      button $logcanvas.log -image Log
-      button $logcanvas.annotate -image Annotate
-      button $logcanvas.diff -image Diff \
+      button $logcanvas.view -image Fileview -highlightbackground $cvsglb(bg)
+      button $logcanvas.log -image Log -highlightbackground $cvsglb(bg)
+      button $logcanvas.annotate -image Annotate -highlightbackground $cvsglb(bg)
+      button $logcanvas.diff -image Diff -highlightbackground $cvsglb(bg) \
         -command [namespace code {
           comparediff_r [$logcanvas.up.revA_rvers cget -text] \
           [$logcanvas.up.revB_rvers cget -text] $logcanvas $filename
         }]
-      button $logcanvas.delta -image Mergediff
-      button $logcanvas.viewtags -image Tags \
+      button $logcanvas.delta -image Mergediff -highlightbackground $cvsglb(bg)
+      button $logcanvas.viewtags -image Tags -highlightbackground $cvsglb(bg) \
         -command [namespace code {
                    variable revtags
                    variable revbtags
@@ -1548,7 +1546,7 @@ namespace eval ::logcanvas {
                    }
                    view_output::new Tags $taglist
                  }]
-      button $logcanvas.close -text "Close" \
+      button $logcanvas.close -text "Close" -highlightbackground $cvsglb(bg) \
         -command [namespace code {
                  global cvscfg
                  variable logcanvas
@@ -1557,7 +1555,7 @@ namespace eval ::logcanvas {
                  namespace delete [namespace current]
                  exit_cleanup 0
                }]
-      button $logcanvas.stop -text "Stop" -bg red4 -fg white \
+      button $logcanvas.stop -text "Stop" -bg red4 -fg white -highlightbackground $cvsglb(bg) \
         -activebackground red4 -activeforeground white \
         -state [expr {$cvscfg(allow_abort) ? {normal} : {disabled}}] \
         -command "$scope\::abortLog"
@@ -1594,7 +1592,7 @@ namespace eval ::logcanvas {
       set_tooltips $logcanvas.diff \
         {"Compare two versions of the file"}
       set_tooltips $logcanvas.delta \
-        {"Merge changes to current"}
+        {"Merge to current"}
       set_tooltips $logcanvas.viewtags \
         {"List all the file\'s tags"}
   
