@@ -136,10 +136,13 @@ proc cvs_workdir_status {} {
         set revision [lindex $line 1]
         regsub {New .*} $revision "New" revision
         set date [lindex $line 2]
+        #puts "from Working Revision: $date"
         # The date field is not supplied to remote clients.
-        if {$date == "" || [string match "New *" $date ] || \
-            [string match "Result *" $date]} {
-          ; # Leave as is
+        if {$date == "" } {
+         if {! ([string match "New *" $date ] || [string match "Result *" $date])} {
+           catch {set date [clock format [file mtime $filename] -format $cvscfg(dateformat)]}
+           gen_log:log E "No date supplied by remote CVS server. Using \[file mtime $filename\]"
+         }
         } else {
           # CVS outputs time strings tcl can't handle, such as
           # ones with +0100.  Let's discard them rather than
@@ -148,11 +151,12 @@ proc cvs_workdir_status {} {
           if {$ret == 0} {
             set juliandate $ans
             set date [clock format $juliandate -format $cvscfg(dateformat)]
-            set Filelist($filename:date) $date
+            #puts "Clock Scan on $ans: $date"
           } else {
             gen_log:log E "$ans"
           }
         }
+        set Filelist($filename:date) $date
         set Filelist($filename:wrev) $revision
         set Filelist($filename:status) $status
       } elseif {[string match "*Sticky Tag:*" $logline]} {
@@ -794,7 +798,7 @@ proc cvs_commit {revision comment args} {
   gen_log:log T "LEAVE"
 }
 
-proc cvs_tag {tagname force branch update args} {
+proc cvs_tag {tagname force b_or_t update args} {
 #
 # This tags a file in a directory.
 #
@@ -802,7 +806,7 @@ proc cvs_tag {tagname force branch update args} {
   global cvscfg
   global incvs
 
-  gen_log:log T "ENTER ($tagname $force $branch $update $args)"
+  gen_log:log T "ENTER ($tagname $force $b_or_t $update $args)"
 
   if {! $incvs} {
     cvs_notincvs
@@ -817,7 +821,7 @@ proc cvs_tag {tagname force branch update args} {
   set filelist [join $args]
 
   set command "$cvs tag"
-  if {$branch == "yes"} {
+  if {$b_or_t == "branch"} {
    append command " -b"
   }
   if {$force == "yes"} {
@@ -825,7 +829,7 @@ proc cvs_tag {tagname force branch update args} {
   }
   append command " $tagname $filelist"
 
-  if {$branch == "yes" && $force == "yes"} {
+  if {$b_or_t == "branch" && $force == "yes"} {
     set too_new 0
     # As of 1.11.2, -F won't move branch tags without the -B option
     set cvsglb(cvs_version) [cvs_version_number]
@@ -861,7 +865,7 @@ proc cvs_tag {tagname force branch update args} {
   gen_log:log T "LEAVE"
 }
 
-proc cvs_update {tagname normal_binary action_if_no_tag get_all_dirs dir args} {
+proc cvs_update {tagname k no_tag recurse prune d dir args} {
 #
 # This updates the files in the current directory.
 #
@@ -869,82 +873,92 @@ proc cvs_update {tagname normal_binary action_if_no_tag get_all_dirs dir args} {
   global cvscfg
   global incvs
 
-  gen_log:log T "ENTER ($tagname $normal_binary $action_if_no_tag $get_all_dirs $dir $args)"
+  gen_log:log T "ENTER (tagname=$tagname k=$k no_tag=$no_tag recurse=$recurse prune=$prune d=$d dir=$dir args=$args)"
 
-  if { $normal_binary == "Normal" } {
-      set mess "Using normal (text) mode.\n"
-  } elseif { $normal_binary == "Binary" } {
-      set mess "Using binary mode.\n"
-  } else {
-      set mess "Unknown mode:  $normal_binary\n"
-  }
+  set filelist [join $args]
 
-  if { $tagname != "BASE"  && $tagname != "HEAD" } {
-      append mess "\nIf a file does not have tag $tagname"
-      if { $action_if_no_tag == "Remove" } {
-          append mess " it will be removed from your local directory.\n"
-      } elseif { $action_if_no_tag == "Get_head" } {
-          append mess " the head revision will be retrieved.\n"
-      } elseif { $action_if_no_tag == "Skip" } {
-          append mess " it will be skipped.\n"
-      }
+  #
+  # cvs update [-APCdflRp] [-k kopt] [-r rev] [-D date] [-j rev]
+  #
+  set commandline "$cvs update"
+
+  if { $k == "Normal" } {
+    set kmsg "\nUsing normal (text) mode."
+  } elseif { $k == "Binary" } {
+    set kmsg "\nUsing binary mode (-kb)."
+    append commandline " -kb"
   }
 
   if { $tagname == "HEAD" } {
     append mess "\nYour local files will be updated to the"
-    append mess " latest main trunk (head) revision."
-    append mess " CVS will try to preserve any local, un-committed changes.\n"
+    append mess " latest main trunk (head) revision (-A)."
+    append commandline " -A"
   }
 
-  append mess "\nIf there is a directory in the repository"
-  append mess " that is not in your local, working directory,"
-  if { $get_all_dirs == "Yes" } {
-    append mess " it will be checked out at this time.\n"
+  if {$recurse == "local"} {
+    append commandline " -l"
   } else {
-    append mess " it will not be checked out.\n"
-  }
-
-  set filelist [join $args]
-  if {$filelist == ""} {
-    append mess "\nYou are about to download from"
-    append mess " the repository to your local"
-    append mess " filespace ** ALL ** files which"
-    append mess " have changed in it."
-  } else {
-    append mess "\nYou are about to download from"
-    append mess " the repository to your local"
-    append mess " filespace these files which"
-    append mess " have changed:\n"
-  
-    foreach file $filelist {
-      append mess "\n\t$file"
+    append mess "\nIf there is a local sub-directory which has"
+    append mess " become empty through deletion of its contents,"
+    if { $prune == "prune" } {
+      append mess " it will be deleted (-P).\n"
+      append commandline " -P"
+    } else {
+      append mess " it will remain.\n"
     }
-  }
-  append mess "\n\nAre you sure?"
-  if {[cvsconfirm $mess .workdir] == "ok"} {
-    # modified by jo to build the commandline incrementally
-    set commandline "$cvs update -P"
-    if { $normal_binary == "Binary" } {
-      append commandline " -kb"
-    }
-    if { $get_all_dirs == "Yes" } {
-      append commandline " -d $dir"
-    }
-    if { $tagname != "BASE" && $tagname != "HEAD" } {
-      if { $action_if_no_tag == "Remove" } {
-          append commandline " -r $tagname"
-      } elseif { $action_if_no_tag == "Get_head" } {
-          append commandline " -f -r $tagname"
-      } elseif { $action_if_no_tag == "Skip" } {
-          append commandline " -s -r $tagname"
+    append mess "\nIf there is a sub-directory in the repository"
+    append mess " that is not here in your local directory,"
+    if { $d == "Yes" } {
+      append mess " it will be checked out at this time (-d).\n"
+      if {$dir != " "} {
+        append mess "($dir only)\n"
       }
+      append commandline " -d $dir"
+    } else {
+      append mess " it will not be checked out.\n"
     }
-    if { $tagname == "HEAD" } {
-      append commandline " -A"
+  }
+
+  if { $tagname != "BASE"  && $tagname != "HEAD" } {
+    append mess "\nYour local files will be updated to the"
+    append mess " tagged revision (-r $tagname)."
+    append mess "  If a file does not have the tag,"
+    if { $no_tag == "Remove" } {
+      append mess " it will be removed from your local directory.\n"
+      append commandline " -r $tagname"
+    } elseif { $no_tag == "Get_head" } {
+      append mess " the head revision will be retrieved.\n"
+      append commandline " -f -r $tagname"
     }
+  }
+
+  if {$filelist == ""} {
+    set filemsg    "\nYou are about to download from"
+    append filemsg " the repository to your local"
+    append filemsg " filespace the files which"
+    append filemsg " are different in the repository,"
+    if {$recurse == "local"} {
+      append filemsg " in this directory only.\n"
+    } else {
+      append filemsg " recursing the sub-directories.\n"
+    }
+  } else {
+    append filemsg "\nYou are about to download from"
+    append filemsg " the repository to your local"
+    append filemsg " filespace these files if they"
+    append filemsg " have changed:\n"
+
     foreach file $filelist {
+      append filemsg "\n\t$file"
       append commandline " \"$file\""
     }
+  }
+  append filemsg "\nIf you have made local changes, they will"
+  append filemsg " be merged into the new local copy.\n"
+  set mess "$filemsg $mess $kmsg"
+  append mess "\n\nAre you sure?"
+
+  if {[cvsconfirm $mess .workdir] == "ok"} {
 
     set co_cmd [viewer::new "CVS Update"]
     $co_cmd\::do $commandline 0 status_colortags
@@ -953,7 +967,7 @@ proc cvs_update {tagname normal_binary action_if_no_tag get_all_dirs dir args} {
   gen_log:log T "LEAVE"
 }
 
-proc cvs_merge {from since fromtag totag args} {
+proc cvs_merge {parent from since frombranch args} {
 #
 # This does a join (merge) of a chosen revision of localfile to the
 # current revision.
@@ -962,52 +976,101 @@ proc cvs_merge {from since fromtag totag args} {
   global cvscfg
   global cvsglb
 
-  gen_log:log T "ENTER (\"$from\" \"$since\" \"$fromtag\" \"$totag\" \"$args\")"
+  gen_log:log T "ENTER (\"$from\" \"$since\" \"$frombranch\" \"$args\")"
 
-  set filelist $args
-  set v [viewer::new "CVS Join"]
+  set filelist [join $args]
 
-  if {$since == ""} {
+  set mergetags [assemble_mergetags $frombranch]
+  set curr_tag [lindex $mergetags 0]
+  set fromtag [lindex $mergetags 1]
+  set totag [lindex $mergetags 2]
+
+  if {$since == {}} {
+    set mess "Merge revision $from\n"
+  } else {
+    set mess "Merge the changes between revision\n $since and $from"
+    append mess " (if $since > $from the changes are removed)\n"
+  }
+  append mess " to the current revision ($curr_tag)"
+  if {[cvsalwaysconfirm $mess $parent] != "ok"} {
+    return
+  }
+
+  # Do the update here, and defer the tagging until later
+  if {$since == {}} {
     set commandline "$cvs update -d -j$from $filelist"
   } else {
     set commandline "$cvs update -d -j$since -j$from $filelist"
   }
-    
-  $v\::do "$commandline"
+  set v [viewer::new "CVS Join"]
+  $v\::do "$commandline" 1 status_colortags
   $v\::wait
 
-  if {$cvscfg(auto_tag)} {
-    set commandline "$cvs tag -F -r $from $fromtag $filelist"
-    $v\::do "$commandline"
-    toplevel .reminder
-    wm title .reminder "Reminder"
-    message .reminder.m1 -aspect 600 -text \
-      "When you are finished checking in your merges, \
-      you should apply the tag"
-    entry .reminder.ent -width 32 -relief groove \
-       -readonlybackground $cvsglb(readonlybg)
-    .reminder.ent insert end $totag 
-    .reminder.ent configure -state readonly
-    message .reminder.m2 -aspect 600 -text \
-      "using the \"Tag the selected files\" button"
-    frame .reminder.bottom -relief raised -bd 2
-    button .reminder.bottom.close -text "Dismiss" \
-      -command {destroy .reminder}
-    pack .reminder.bottom -side bottom -fill x
-    pack .reminder.bottom.close -side bottom -expand yes
-    pack .reminder.m1 -side top
-    pack .reminder.ent -side top -padx 2
-    pack .reminder.m2 -side top
-  }
-
   if [winfo exists .workdir] {
     if {$cvscfg(auto_status)} {
       setup_dir
     }
+  } else {
+    workdir_setup
   }
+
+  dialog_merge_notice cvs $from $frombranch $fromtag $totag $filelist
+
   gen_log:log T "LEAVE"
 }
 
+proc cvs_merge_tag_seq {from frombranch totag fromtag args} {
+  global cvs
+  global cvscfg
+
+  gen_log:log T "ENTER (\"$from\" \"$totag\" \"$fromtag\" $args)"
+
+  set filelist ""
+  foreach f $args {
+    append filelist "\"$f\" "
+  }
+
+  # It's muy importante to make sure everything is OK at this point
+  set commandline "$cvs -n -q update $filelist"
+  gen_log:log C "$commandline"
+  set ret [catch {eval "exec $commandline"} view_this]
+  set logmode [expr {$ret ? {E} : {D}}]
+  view_output::new "CVS Check" $view_this
+  gen_log:log $logmode $view_this
+  if {$ret} {
+    set mess "CVS Check shows errors which would prevent a successful\
+    commit. Please resolve them before continuing."
+    if {[cvsalwaysconfirm $mess .workdir] != "ok"} {
+      return
+    }
+  }
+  # Do the commit
+  set v [viewer::new "CVS Commit and Tag a Merge"]
+  $v\::log "$cvs commit -m \"Merge from $from\" $filelist\n"
+  $v\::do "$cvs commit -m \"Merge from $from\" $filelist" 1
+  $v\::wait
+  # Tag if desired
+  if {$cvscfg(auto_tag) && $totag != ""} {
+    # First, the "from" file that's not in this branch (needs -r)
+    set commandline "$cvs tag -F -r$from $totag $filelist"
+    $v\::log "$commandline\n"
+    $v\::do "$commandline" 1
+    $v\::wait
+  }
+  if {$cvscfg(auto_tag) && $fromtag != ""} {
+    # Now, the version that's in the current branch
+    set commandline "$cvs tag -F $fromtag $filelist"
+    $v\::log "$commandline\n"
+    $v\::do "$commandline" 1
+    $v\::wait
+  } 
+  catch {destroy .reminder}
+
+  if {$cvscfg(auto_status)} {
+    setup_dir
+  }
+}
+
 proc cvs_status {args} {
 #
 # This does a status report on the files in the current directory.
@@ -1028,11 +1091,10 @@ proc cvs_status {args} {
   busy_start .workdir.main
   set filelist [join $args]
   # if recurse option is true or there are no selected files, recurse
-  set cmd_options ""
-  if {! [info exists cvscfg(recurse)]} {
-    set cmd_options "-l"
+  set flags ""
+  if {$cvscfg(recurse)} {
+    set flags "-l"
   }
-
   # if there are selected files, use verbose output
   # but save the current setting so it can be reset
   # - added by Jo
@@ -1043,7 +1105,7 @@ proc cvs_status {args} {
   }
 
   # support verious levels of verboseness. Ideas derived from GIC
-  set statcmd [exec::new "$cvs -Q status $cmd_options $filelist"]
+  set statcmd [exec::new "$cvs -Q status $flags $filelist"]
   set raw_status [$statcmd\::output]
   $statcmd\::destroy
 
@@ -1317,8 +1379,9 @@ proc cvs_merge_conflict {args} {
     regsub -all {\$} $file {\$} filename
     set commandline "$cvs -n -q update \"$filename\""
     gen_log:log C "$commandline"
-    catch {eval "exec $commandline"} status
-    gen_log:log C "$status"
+    set ret [catch {eval "exec $commandline"} status]
+    set logmode [expr {$ret ? {E} : {D}}]
+    gen_log:log $logmode "$status"
 
     gen_log:log F "OPEN $file"
     set f [open $file]
@@ -1338,14 +1401,15 @@ proc cvs_merge_conflict {args} {
       # we can resolve the conflict
       gen_log:log C "$commandline"
       set commandline "$cvs update \"$file\""
-      gen_log:log C "$status"
-      catch {eval "exec $commandline"} status
+      set ret [catch {eval "exec $commandline"} status]
+      set logmode [expr {$ret ? {E} : {D}}]
+      gen_log:log $logmode "$status"
     } elseif { $match == 1 } { 
       # There are conflict markers already, dont update
       ;
     } else {
-      cvsfail "This file does not appear to have a conflict." .workdir
-      return
+      cvsfail "$file does not appear to have a conflict." .workdir
+      continue
     }
     # Invoke tkdiff with the proper option for a conflict file
     # and have it write to the original file
@@ -1406,41 +1470,45 @@ proc cvs_gettaglist {filename parent} {
   return "$keepers"
 }
 
-proc cvs_release {delflag directory} {
+proc cvs_release {delflag args} {
   global cvs
   global cvscfg
 
-  gen_log:log T "ENTER ($directory)"
-  if {! [file isdirectory $directory]} {
-    cvsfail "$directory is not a directory" .workdir
-    return
-  }
+  gen_log:log T "ENTER ($args)"
+  set filelist [join $args]
 
-  set commandline "$cvs -n -q update $directory"
-  gen_log:log C "$commandline"
-  set ret [catch {eval "exec $commandline"} view_this]
-  if {$view_this != ""} {
-    view_output::new "CVS Check" $view_this
-    set mess "$directory is not up-to-date."
-    append mess "\nRelease anyway?"
-    if {[cvsconfirm $mess .workdir] != "ok"} {
+  foreach directory $filelist {
+    if {! [file isdirectory $directory]} {
+      cvsfail "$directory is not a directory" .workdir
       return
     }
+  
+    set commandline "$cvs -n -q update \"$directory\""
+    gen_log:log C "$commandline"
+    set ret [catch {eval "exec $commandline"} view_this]
+    if {$view_this != ""} {
+      view_output::new "CVS Check" $view_this
+      set mess "\"$directory\" is not up-to-date."
+      append mess "\nRelease anyway?"
+      if {[cvsconfirm $mess .workdir] != "ok"} {
+        return
+      }
+    }
+    set commandline "$cvs -Q release $delflag \"$directory\""
+    set ret [catch {eval "exec $commandline"} view_this]
+    gen_log:log C "$commandline"
+    if {$ret != 0} {
+      view_output::new "CVS Release" $view_this
+    }
   }
-  set commandline "$cvs -Q release $delflag $directory"
-  set ret [catch {eval "exec $commandline"} view_this]
-  gen_log:log C "$commandline"
-  if {$ret != 0} {
-    view_output::new "CVS Release" $view_this
-  }
-
+  
   if {$cvscfg(auto_status)} {
     setup_dir
   }
   gen_log:log T "LEAVE"
 }
 
-proc cvs_rtag { cvsroot mcode branch force oldtag newtag } {
+proc cvs_rtag { cvsroot mcode b_or_t force oldtag newtag } {
 #
 # This tags a module in the repository.
 # Called by the tag commands in the Repository Browser
@@ -1448,14 +1516,14 @@ proc cvs_rtag { cvsroot mcode branch force oldtag newtag } {
   global cvs
   global cvscfg
   
-  gen_log:log T "ENTER ($cvsroot $mcode $branch $force $oldtag $newtag)"
+  gen_log:log T "ENTER ($cvsroot $mcode $b_or_t $force $oldtag $newtag)"
   if {$newtag == ""} {
     cvsfail "You must enter a tag name!" .modbrowse
     return 1
   }
 
   set command "$cvs -d \"$cvsroot\" rtag"
-  if {$branch == "yes"} {
+  if {$b_or_t == "branch"} {
     append command " -b"
   } 
   if {$force == "yes"} {
@@ -1550,24 +1618,24 @@ proc cvs_commit_dialog {} {
   pack .commit.message -in .commit.top -padx 2 -pady 5
 
 
-  button .commit.ok -text "OK" \
+  button .commit.ok -text "OK" -highlightbackground $cvsglb(bg) \
     -command {
       grab release .commit
       wm withdraw .commit
       set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
       cvs_commit $version $cvsglb(commit_comment) $cvsglb(commit_list)
     }
-  button .commit.apply -text "Apply" \
+  button .commit.apply -text "Apply" -highlightbackground $cvsglb(bg) \
     -command {
       set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
       cvs_commit $version $cvsglb(commit_comment) $cvsglb(commit_list)
     }
-  button .commit.clear -text "ClearAll" \
+  button .commit.clear -text "ClearAll" -highlightbackground $cvsglb(bg) \
     -command {
       set version ""
       .commit.tcomment delete 1.0 end
     }
-  button .commit.quit \
+  button .commit.quit -highlightbackground $cvsglb(bg) \
     -command {
       grab release .commit
       wm withdraw .commit
@@ -1691,6 +1759,7 @@ proc read_cvs_dir {dirname} {
   global module_dir
   global cvscfg
   global cvsglb
+  global cvs
   global current_tagname
 
   gen_log:log T "ENTER ($dirname)"
@@ -1723,14 +1792,27 @@ proc read_cvs_dir {dirname} {
       }
     } else {
       cvsfail "Repository file not found in $dirname" .workdir
+      return 0
     }
   } else {
     cvsfail "$dirname is not a directory" .workdir
+    return 0
   }
   set cvsglb(root) $cvscfg(cvsroot)
+  # This confirms whether we can really use cvs but do we
+  # want to waste time doing it?
   #gen_log:log D "cvsglb(root) $cvsglb(root)"
   #gen_log:log D "cvscfg(cvsroot) $cvscfg(cvsroot)"
-  gen_log:log T "LEAVE"
+  #set command "$cvs version"
+  #gen_log:log C "$command"
+  #set ret [catch {eval "exec $command"} output]
+  #if {$ret} {
+    #cvsfail $output
+    #return 0
+  #}
+
+  gen_log:log T "LEAVE (1)"
+  return 1
 }
 
 proc parse_cvsmodules {modules_file} {
@@ -2059,6 +2141,7 @@ namespace eval ::cvs_branchlog {
                   set localfile [string range $logline 14 end]
                 #}
               } elseif {$logline == "symbolic names:"} {
+                # FIXME: old RCS can have a tag on this line
                 set logstate {T}
               }
             }
