@@ -18,16 +18,19 @@ proc gen_log:init {} {
       -command ".trace.text yview"
   frame .trace.bottom
 
-  button .trace.bottom.clear -text "Clear" -command gen_log:clear
+  button .trace.bottom.clear -text "Clear" -highlightbackground $cvsglb(bg) \
+    -command gen_log:clear
+  button .trace.bottom.save -text "Save to File" -highlightbackground $cvsglb(bg) \
+    -command gen_log:save
 
   search_textwidget_init
-  button .trace.bottom.srchbtn -text Search \
+  button .trace.bottom.srchbtn -text Search -highlightbackground $cvsglb(bg) \
     -command "search_textwidget .trace.text"
   entry .trace.bottom.entry -width 20 -textvariable cvsglb(searchstr)
   bind .trace.bottom.entry <Return> \
       "search_textwidget .trace.text"
 
-  button .trace.close -text "Stop Tracing" \
+  button .trace.close -text "Stop Tracing" -highlightbackground $cvsglb(bg) \
     -command { gen_log:quit; exit_cleanup 0 }
 
   pack .trace.bottom -side bottom -fill x
@@ -37,6 +40,7 @@ proc gen_log:init {} {
   pack .trace.bottom.srchbtn -side left
   pack .trace.bottom.entry -side left
   pack .trace.bottom.clear -side left -expand 1 -anchor c
+  pack .trace.bottom.save -side left
   pack .trace.close -in .trace.bottom -side right
 
   # Lets hard-code these colors - otherwise at least one is very
@@ -93,6 +97,28 @@ proc gen_log:clear { } {
    .trace.text delete 1.0 end
 }
 
+proc gen_log:save { } {
+  set initialfile "tkcvs_log.txt"
+
+  set types  { {{All Files} *} }
+  set savfile [ \
+    tk_getSaveFile -title "Save Trace" \
+       -filetypes $types \
+       -initialfile $initialfile \
+       -parent .trace
+  ]
+  if {$savfile == ""} {
+    return
+  }
+
+  if {[catch {set fo [open $savfile w]}]} {
+    catchputs "Cannot open $savfile for writing"
+    return
+  }
+  puts $fo [.trace.text get 1.0 end]
+  close $fo
+}
+
 proc gen_log:changeclass { } {
   global cvscfg
   global logclass
