@@ -542,8 +542,10 @@ namespace eval joincanvas {
       frame $joincanvas.up -relief groove -border 2
       pack $joincanvas.up -side top -fill x
 
-      button $joincanvas.up.bworkdir -image Workdir -command { workdir_setup }
-      button $joincanvas.up.bmodbrowse -image Modules_cvs -command { modbrowse_run cvs }
+      button $joincanvas.up.bworkdir -image Workdir -highlightbackground $cvsglb(bg) \
+       -command { workdir_setup }
+      button $joincanvas.up.bmodbrowse -image Modules_cvs -highlightbackground $cvsglb(bg) \
+       -command { modbrowse_run cvs }
 
       label $joincanvas.up.lfname -text "Representative File" -anchor w
       entry $joincanvas.up.rfname -textvariable [namespace current]::repfile
@@ -603,25 +605,21 @@ namespace eval joincanvas {
       #
       # Create buttons
       #
-      button $joincanvas.join -image Mergebranch \
-          -command [namespace code {
-                   set fromrev [$joincanvas.up.rversFrom get]
-                   merge_dialog CVS \
-                     $fromrev "" $fromrev .
-                 }]
-      button $joincanvas.delta -image Mergediff \
+      button $joincanvas.delta -image Mergediff -highlightbackground $cvsglb(bg) \
           -command [namespace code {
                  set fromrev [$joincanvas.up.rversFrom get]
+                 if {$fromrev == ""} {
+                   cvsfail "Please select a branch!" $joincanvas; return
+                 }
                  set sincerev [$joincanvas.up.rversSince.e get]
-                 merge_dialog CVS \
-                   $fromrev $sincerev $fromrev .
+                 cvs_merge $joincanvas $fromrev $sincerev $fromrev .
                  }]
 
-      button $joincanvas.down.blogfile -image Branches \
+      button $joincanvas.down.blogfile -image Branches -highlightbackground $cvsglb(bg) \
          -command "cvs_branches $repfile"
       frame $joincanvas.down.btnfm
       frame $joincanvas.down.closefm -relief groove -bd 2
-      button $joincanvas.close -text "Close" \
+      button $joincanvas.close -text "Close" -highlightbackground $cvsglb(bg) \
         -command [namespace code "
                    destroy $joincanvas
                    namespace delete [namespace current]
@@ -631,8 +629,7 @@ namespace eval joincanvas {
       pack $joincanvas.down.blogfile -side left \
         -ipadx 4 -ipady 4
       pack $joincanvas.down.btnfm -side left -fill y -expand 1
-      pack $joincanvas.join \
-           $joincanvas.delta \
+      pack $joincanvas.delta \
         -in $joincanvas.down.btnfm -side left \
         -ipadx 4 -ipady 4
       pack $joincanvas.down.closefm -side right
@@ -642,10 +639,8 @@ namespace eval joincanvas {
 
       set_tooltips $joincanvas.down.blogfile \
          {"Revision Log and Branch Diagram of the current file"}
-      set_tooltips $joincanvas.join \
-         {"Merge to current"}
       set_tooltips $joincanvas.delta \
-         {"Merge changes to current"}
+         {"Merge to current"}
       set_tooltips $joincanvas.up.bworkdir \
         {"Open the Working Directory Browser"}
       set_tooltips $joincanvas.up.bmodbrowse \
