@@ -12,7 +12,7 @@ proc modbrowse_setup {} {
   global cvscfg
   global tcl_platform
 
-  gen_log:log T "ENTER"
+  #gen_log:log T "ENTER"
   set cwd [pwd]
 
   # Window manager stuff.
@@ -52,7 +52,8 @@ proc modbrowse_setup {} {
   ::picklist::bind .modbrowse.top.troot <Return> \
     {if {$repository_root != $cvsglb(root)} {modbrowse_run}}
 
-  button .modbrowse.top.bworkdir -image Workdir -command {workdir_setup}
+  button .modbrowse.top.bworkdir -image Workdir -highlightbackground $cvsglb(bg) \
+    -command {workdir_setup}
 
   label .modbrowse.top.lcwd -text "Current Directory"
   ::picklist::entry .modbrowse.top.tcwd cwd directory
@@ -88,36 +89,36 @@ proc modbrowse_setup {} {
   #
   # Create buttons
   #
-  button .modbrowse.bottom.buttons.modfuncs.filebrowse -image Files \
+  button .modbrowse.bottom.buttons.modfuncs.filebrowse -image Files -highlightbackground $cvsglb(bg) \
     -command { browse_files $modbrowse_module }
-  button .modbrowse.bottom.buttons.modfuncs.patchsummary -image Patches \
+  button .modbrowse.bottom.buttons.modfuncs.patchsummary -image Patches -highlightbackground $cvsglb(bg) \
     -command { dialog_cvs_patch $cvscfg(cvsroot) $modbrowse_module 1 }
-  button .modbrowse.bottom.buttons.modfuncs.patchfile -image Patchfile \
+  button .modbrowse.bottom.buttons.modfuncs.patchfile -image Patchfile -highlightbackground $cvsglb(bg) \
     -command { dialog_cvs_patch $cvscfg(cvsroot) $modbrowse_module 0 }
-  button .modbrowse.bottom.buttons.modfuncs.checkout -image Checkout \
+  button .modbrowse.bottom.buttons.modfuncs.checkout -image Checkout -highlightbackground $cvsglb(bg) \
     -command { dialog_cvs_checkout $cvscfg(cvsroot) $modbrowse_module }
-  button .modbrowse.bottom.buttons.modfuncs.export -image Export \
+  button .modbrowse.bottom.buttons.modfuncs.export -image Export -highlightbackground $cvsglb(bg) \
     -command { dialog_cvs_export $cvscfg(cvsroot) $modbrowse_module }
-  button .modbrowse.bottom.buttons.modfuncs.tag -image Tag \
-    -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "no" }
-  button .modbrowse.bottom.buttons.modfuncs.branchtag -image Branchtag \
-    -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "yes" }
+  button .modbrowse.bottom.buttons.modfuncs.tag -image Tag -highlightbackground $cvsglb(bg) \
+    -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "tag" }
+  button .modbrowse.bottom.buttons.modfuncs.branchtag -image Branchtag -highlightbackground $cvsglb(bg) \
+    -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "branch" }
 
-  button .modbrowse.bottom.buttons.svnfuncs.filecat -image Fileview \
+  button .modbrowse.bottom.buttons.svnfuncs.filecat -image Fileview -highlightbackground $cvsglb(bg) \
     -command { svn_filecat $cvscfg(svnroot) $modbrowse_path $modbrowse_title}
-  button .modbrowse.bottom.buttons.svnfuncs.filelog -image Log \
+  button .modbrowse.bottom.buttons.svnfuncs.filelog -image Log -highlightbackground $cvsglb(bg) \
     -command { svn_filelog $cvscfg(svnroot) $modbrowse_path $modbrowse_title}
-  button .modbrowse.bottom.buttons.svnfuncs.remove -image SvnRemove \
+  button .modbrowse.bottom.buttons.svnfuncs.remove -image SvnRemove -highlightbackground $cvsglb(bg) \
     -command { svn_delete $cvscfg(svnroot) $modbrowse_path }
 
-  button .modbrowse.bottom.buttons.cvsfuncs.import -image Import \
+  button .modbrowse.bottom.buttons.cvsfuncs.import -image Import -highlightbackground $cvsglb(bg) \
      -command { import_run }
-  button .modbrowse.bottom.buttons.cvsfuncs.who -image Who \
+  button .modbrowse.bottom.buttons.cvsfuncs.who -image Who -highlightbackground $cvsglb(bg) \
      -command {cvs_history all $modbrowse_module}
-  button .modbrowse.bottom.buttons.cvsfuncs.brefresh  -image Refresh \
+  button .modbrowse.bottom.buttons.cvsfuncs.brefresh  -image Refresh -highlightbackground $cvsglb(bg) \
      -command { modbrowse_run }
 
-  button .modbrowse.bottom.buttons.closefm.close -text "Close" \
+  button .modbrowse.bottom.buttons.closefm.close -text "Close" -highlightbackground $cvsglb(bg) \
     -command { module_exit; exit_cleanup 0 }
 
   grid .modbrowse.bottom.buttons.cvsfuncs.brefresh -column 0 -row 0 \
@@ -193,7 +194,7 @@ proc modbrowse_setup {} {
   wm maxsize .modbrowse $screenWidth $screenHeight
   wm minsize .modbrowse 430 300
 
-  gen_log:log T "LEAVE"
+  #gen_log:log T "LEAVE"
 }
 
 proc modbrowse_images {} {
@@ -218,10 +219,11 @@ proc modbrowse_images {} {
 
 proc modbrowse_menus {} {
   global cvscfg
+  global cvsglb
   global cvs
   global logclass
 
-  gen_log:log T "ENTER"
+  #gen_log:log T "ENTER"
 
   menu .modbrowse.modmenu
   .modbrowse configure -menu .modbrowse.modmenu
@@ -241,7 +243,6 @@ proc modbrowse_menus {} {
   #
   # Create the menus
   #
-  set selcolor [option get .modbrowse selectColor selectColor]
   .modbrowse.modmenu.file add command -label "Browse Working Directory" -underline 0 \
      -command workdir_setup
   .modbrowse.modmenu.file add separator
@@ -255,9 +256,9 @@ proc modbrowse_menus {} {
   .modbrowse.modmenu.cvs add command -label "CVS Export" \
       -command { dialog_cvs_export $cvscfg(cvsroot) $modbrowse_module}
   .modbrowse.modmenu.cvs add command -label "Tag Module" -underline 0 \
-     -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "no" }
+     -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "tag" }
   .modbrowse.modmenu.cvs add command -label "Branch Tag Module" -underline 0 \
-     -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "yes" }
+     -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "branch" }
   .modbrowse.modmenu.cvs add command -label "Make Patch File" -underline 0 \
      -command { dialog_cvs_patch $cvscfg(cvsroot) $modbrowse_module 0 }
   .modbrowse.modmenu.cvs add command -label "View Patch Summary" -underline 0 \
@@ -282,9 +283,9 @@ proc modbrowse_menus {} {
   .modbrowse.modmenu.svn add command -label "SVN Export" \
       -command { dialog_svn_checkout $cvscfg(svnroot) $modbrowse_path export}
   .modbrowse.modmenu.svn add command -label "Tag Module" -underline 0 \
-     -command { dialog_svn_copy $cvscfg(svnroot) $modbrowse_path "tags" }
+     -command { dialog_svn_tag $cvscfg(svnroot) $modbrowse_path "tags" }
   .modbrowse.modmenu.svn add command -label "Branch Module" -underline 0 \
-     -command { dialog_svn_copy $cvscfg(svnroot) $modbrowse_path "branches" }
+     -command { dialog_svn_tag $cvscfg(svnroot) $modbrowse_path "branches" }
   .modbrowse.modmenu.svn add command -label "Make Patch File" -underline 0 \
      -command { dialog_svn_patch $cvscfg(cvsroot) $modbrowse_path $selB_path 0 }
   .modbrowse.modmenu.svn add command -label "View Patch Summary" -underline 0 \
@@ -295,7 +296,7 @@ proc modbrowse_menus {} {
 
   .modbrowse.modmenu.options add checkbutton -label "Group Aliases in a Folder (CVS)" \
      -variable cvscfg(aliasfolder) -onvalue true -offvalue false \
-     -selectcolor $selcolor -command {
+     -command {
         ModTree:delitem .modbrowse.treeframe /
         ModTree:destroy .modbrowse.treeframe
         busy_start .modbrowse
@@ -306,30 +307,30 @@ proc modbrowse_menus {} {
   .modbrowse.modmenu.options add separator
   .modbrowse.modmenu.options add checkbutton -label "Tracing On/Off" \
      -variable cvscfg(logging) -onvalue true -offvalue false \
-     -selectcolor $selcolor -command log_toggle
+     -command log_toggle
   .modbrowse.modmenu.options add cascade -label "Trace Level" \
      -menu .modbrowse.modmenu.options.loglevel
   menu .modbrowse.modmenu.options.loglevel
   .modbrowse.modmenu.options.loglevel add checkbutton -label "CVS commands (C)" \
      -variable logclass(C) -onvalue "C" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .modbrowse.modmenu.options.loglevel add checkbutton -label "CVS stderr (E)" \
      -variable logclass(E) -onvalue "E" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .modbrowse.modmenu.options.loglevel add checkbutton -label "File creation/deletion (F)"\
      -variable logclass(F) -onvalue "F" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .modbrowse.modmenu.options.loglevel add checkbutton -label "Function entry/exit (T)" \
      -variable logclass(T) -onvalue "T" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .modbrowse.modmenu.options.loglevel add checkbutton -label "Debugging (D)" \
      -variable logclass(D) -onvalue "D" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
 
 
   menu_std_help .modbrowse.modmenu
 
-  gen_log:log T "LEAVE"
+  #gen_log:log T "LEAVE"
 }
 
 proc modbrowse_run { {CVSorSVN {}} } {
@@ -390,7 +391,7 @@ proc modbrowse_run { {CVSorSVN {}} } {
 
       set cvsglb(root) $cvscfg(cvsroot)
       set cmd(cvs_co) \
-          [exec::new "$cvs -d $cvscfg(cvsroot) checkout -p CVSROOT/modules"]
+          [exec::new "$cvs -d \"$cvscfg(cvsroot)\" checkout -p CVSROOT/modules"]
       .modbrowse.top.lroot configure -text "CVSROOT"
       .modbrowse.top.lmcode configure -text "Module"
       ModTree:create .modbrowse.treeframe
@@ -425,7 +426,7 @@ proc modbrowse_run { {CVSorSVN {}} } {
 
         #set cvsglb(root) $cvscfg(cvsroot)
         set cmd(cvs_co) \
-            [exec::new "$cvs -d $cvscfg(cvsroot) checkout -p CVSROOT/modules"]
+            [exec::new "$cvs -d \"$cvscfg(cvsroot)\" checkout -p CVSROOT/modules"]
         .modbrowse.top.lroot configure -text "CVSROOT"
         .modbrowse.top.lmcode configure -text "Module"
         ModTree:create .modbrowse.treeframe
@@ -455,9 +456,9 @@ proc modbrowse_run { {CVSorSVN {}} } {
     .modbrowse.bottom.buttons.modfuncs.export configure -state normal \
       -command { dialog_svn_checkout $cvscfg(svnroot) $modbrowse_path export}
     .modbrowse.bottom.buttons.modfuncs.tag configure -state normal \
-      -command { dialog_svn_copy $cvscfg(svnroot) $modbrowse_path "tags" }
+      -command { dialog_svn_tag $cvscfg(svnroot) $modbrowse_path "tags" }
     .modbrowse.bottom.buttons.modfuncs.branchtag configure -state normal \
-      -command { dialog_svn_copy $cvscfg(svnroot) $modbrowse_path "branches" }
+      -command { dialog_svn_tag $cvscfg(svnroot) $modbrowse_path "branches" }
     .modbrowse.bottom.buttons.modfuncs.patchsummary configure -state normal \
       -command { dialog_svn_patch $cvscfg(svnroot) $modbrowse_path $selB_path 1 }
     .modbrowse.bottom.buttons.modfuncs.patchfile configure -state normal \
@@ -477,9 +478,9 @@ proc modbrowse_run { {CVSorSVN {}} } {
     .modbrowse.bottom.buttons.modfuncs.export configure -state normal \
       -command { dialog_cvs_export $cvscfg(cvsroot) $modbrowse_module }
     .modbrowse.bottom.buttons.modfuncs.tag configure -state normal \
-      -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "no" }
+      -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "tag" }
     .modbrowse.bottom.buttons.modfuncs.branchtag configure -state normal \
-      -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "yes" }
+      -command { rtag_dialog $cvscfg(cvsroot) $modbrowse_module "branch" }
     .modbrowse.bottom.buttons.modfuncs.patchsummary configure -state normal \
       -command { dialog_cvs_patch $cvscfg(cvsroot) $modbrowse_module 1 }
     .modbrowse.bottom.buttons.modfuncs.patchfile configure -state normal \
