@@ -8,7 +8,6 @@
 #
 proc ModTree:create {w {open_func {}} } {
   global Tree
-  global cvscfg
   global cvsglb
 
   gen_log:log T "ENTER ($w $open_func)"
@@ -19,27 +18,17 @@ proc ModTree:create {w {open_func {}} } {
   }
 
   set winwid [winfo width $w]
-  panedwindow $w.pw -bg $cvsglb(canvbg) -relief sunk -bd 2
+  panedwindow $w.pw -relief sunk -bd 2
   $w.pw configure -handlepad 35 -sashwidth 4 -sashpad 0 -handlesize 10
-  frame $w.tree -bg $cvsglb(canvbg)
-  frame $w.labl -bg $cvsglb(canvbg)
+  frame $w.tree
+  frame $w.labl
 
   canvas $w.tree.list -highlightthickness 0 -width [expr {$winwid * 3/8}]
   canvas $w.labl.list -highlightthickness 0
-
-  set cvsglb(fg) [lindex [.modbrowse.bottom.buttons.modfuncs.filebrowse configure -foreground] 4]
-  set cvsglb(dfg) \
-      [lindex [.modbrowse.top.bworkdir configure -disabledforeground] 4]
-  set buttonhilite \
-      [lindex [.modbrowse.top.bworkdir configure -highlightbackground] 4]
-  set selcolor [option get . selectColor selectColor]
-
-  if {[string length $selcolor]} {
-    set cvsglb(hlbg) $selcolor
-  }
-  if {$cvsglb(hlbg) == $cvsglb(canvbg)} {
-    set cvsglb(hlbg) $buttonhilite
-  }
+  $w.tree configure -bg $cvsglb(canvbg)
+  $w.labl configure -bg $cvsglb(canvbg)
+  $w.tree.list configure -bg $cvsglb(canvbg)
+  $w.labl.list configure -bg $cvsglb(canvbg)
 
   scrollbar $w.yscroll -orient vertical -highlightthickness 0 \
       -command "ModTree:scroll_windows $w"
@@ -244,7 +233,7 @@ proc ModTree:buildlayer {w v in} {
   global cvscfg
   global cvsglb
 
-  gen_log:log T "ENTER ($w $v $in)"
+  #gen_log:log T "ENTER ($w $v $in)"
   if {$v=="/"} {
     set vx {}
   } else {
@@ -292,26 +281,36 @@ proc ModTree:buildlayer {w v in} {
     $w.tree.list bind $w.tree.list.tx$j <1> "ModTree:setselection $w \"$fn\""
     $w.tree.list bind $w.tree.list.tx$j <2> "ModTree:setselB $w \"$fn\""
     $w.tree.list bind $w.tree.list.tx$j <3> "ModTree:setselB $w \"$fn\""
-    $w.tree.list bind $w.tree.list.tx$j <Enter> "ModTree:flash $w \"$fn\""
-    $w.tree.list bind $w.tree.list.tx$j <Leave> "ModTree:unflash $w \"$fn\""
+    $w.tree.list bind $w.tree.list.tx$j <Enter> "ModTree:flash $w $j"
+    $w.tree.list bind $w.tree.list.tx$j <Leave> "ModTree:unflash $w $j"
 
     #gen_log:log D "$vx/$c $lbl   j=$j"
     if {[info exists Tree($w:$vx/$c:title)]} {
       set k [$w.labl.list create text [expr {$x - $Tree(vsize) - 22}] $y \
          -text $Tree($w:$vx/$c:title) \
          -fill $cvsglb(fg) \
-         -font $cvscfg(listboxfont) -anchor w]
+         -font $cvscfg(listboxfont) -anchor w \
+         -tag $w.labl.list.tx$j]
     }
+    $w.labl.list bind $w.labl.list.tx$j <1> "ModTree:setselection $w \"$fn\""
+    $w.labl.list bind $w.labl.list.tx$j <2> "ModTree:setselB $w \"$fn\""
+    $w.labl.list bind $w.labl.list.tx$j <3> "ModTree:setselB $w \"$fn\""
+    $w.labl.list bind $w.labl.list.tx$j <Enter> "ModTree:flash $w $j"
+    $w.labl.list bind $w.labl.list.tx$j <Leave> "ModTree:unflash $w $j"
+
+
     set Tree($w:tag:$j) $vx/$c
     set Tree($w:$vx/$c:tag) $j
     # Put an open/close image on it if it has children
     if {[string length $Tree($w:$vx/$c:children)]} {
       if {$Tree($w:$vx/$c:open)} {
+         # It's closed.  Draw a plus sign and bind a function to close the sub-tree
          set k [$w.tree.list create image $in $y -image ModTree:openbm]
          $w.tree.list bind $k <1> "set \"Tree($w:$vx/$c:open)\" 0; \
                                    ModTree:build $w"
          ModTree:buildlayer $w $vx/$c [expr {$in+$Tree(vsize)+8}]
       } else {
+         # It's open.  Draw a minus sign and bind a function to build the sub-tree
          set k [$w.tree.list create image $in $y -image ModTree:closedbm]
          if {$Tree(open_function) == {} } {
            $w.tree.list bind $k <1> "set \"Tree($w:$vx/$c:open)\" 1; \
@@ -379,7 +378,7 @@ proc ModTree:setselection {w v} {
   set oldv $Tree($w:selection)
   if {$oldv != ""} {
     set j $Tree($w:$oldv:tag)
-    ModTree:clearTextHBox $w $w.tree.list.tx$j
+    ModTree:clearTextHBox $w $j
   }
   #foreach a [array names Tree "$w:$v:*"] { puts "$a $Tree($a)" }
 
@@ -387,11 +386,12 @@ proc ModTree:setselection {w v} {
   if {$v != ""} {
     set Tree($w:selection) $v
     set j $Tree($w:$v:tag)
-    ModTree:setTextHBox $w $w.tree.list.tx$j
+    #ModTree:setTextHBox $w $w.tree.list.tx$j
+    ModTree:setTextHBox $w $j
     set modbrowse_module $Tree($w:$v:name)
     set modbrowse_title $Tree($w:$v:title)
   }
-  set modbrowse_path $v
+  set modbrowse_path [string trimleft $v /]
 }
 
 #
@@ -405,14 +405,15 @@ proc ModTree:setselB {w v} {
   set oldv $Tree($w:selB)
   if {$oldv != ""} {
     set j $Tree($w:$oldv:tag)
-    ModTree:clearTextHBox $w $w.tree.list.tx$j
+    ModTree:clearTextHBox $w $j
   }
 
   # Hilight new selection
   if {$v != ""} {
     set Tree($w:selB) $v
     set j $Tree($w:$v:tag)
-    ModTree:setTextHBox $w $w.tree.list.tx$j
+    #ModTree:setTextHBox $w $w.tree.list.tx$j
+    ModTree:setTextHBox $w $j
   }
   set selB_path $v
 }
@@ -432,22 +433,18 @@ proc ModTree:clearselection {w} {
   }
 }
 
-proc ModTree:flash {widg v} {
-  global Tree
+proc ModTree:flash {w y} {
+  global cvscfg
 
-  set j $Tree($widg:$v:tag)
-  ModTree:setTextHBox $widg $widg.tree.list.tx$j
+  $w.tree.list itemconfigure $w.tree.list.tx$y -font $cvscfg(flashfont)
+  $w.labl.list itemconfigure $w.labl.list.tx$y -font $cvscfg(flashfont)
 }
 
-proc ModTree:unflash {widg v} {
-  global Tree
+proc ModTree:unflash {w y} {
+  global cvscfg
 
-  set j $Tree($widg:$v:tag)
-
-  # Don't unflash if this is one that is selected:
-  if { $Tree($widg:selection) != $v && $Tree($widg:selB) != $v } {
-  	ModTree:clearTextHBox $widg $widg.tree.list.tx$j
-  }
+  $w.tree.list itemconfigure $w.tree.list.tx$y -font $cvscfg(listboxfont)
+  $w.labl.list itemconfigure $w.labl.list.tx$y -font $cvscfg(listboxfont)
 }
 
 proc ModTree:scroll_windows {w args} {
@@ -490,20 +487,33 @@ proc ModTree:clearTextHBox {w id} {
    global cvsglb
 
     # clear the tag corresponding to the text label
-    catch {$w.tree.list delete HBox$id}
-    $w.tree.list itemconfigure $id -fill $cvsglb(fg) 
+    catch {$w.tree.list delete HBox$w.tree.list.tx$id}
+    $w.tree.list itemconfigure $w.tree.list.tx$id -fill $cvsglb(fg) 
+    catch {$w.labl.list delete HBox$w.tree.list.tx$id}
+    $w.labl.list itemconfigure $w.labl.list.tx$id -fill $cvsglb(fg) 
 }
 
 # set a text highligh box (used by set/clearselection)
 proc ModTree:setTextHBox {w id} {
    global cvsglb
-   
-   # get the bounding box for the text id
-   set bbox [$w.tree.list bbox $id]
-   if {[llength $bbox]==4} {
-    # create rectangle with fill, tagged with the same ID as the text, so we can delete it later
-    set i [eval $w.tree.list create rectangle $bbox -fill $cvsglb(hlbg) -tag HBox$id -outline \"\"] 
-    $w.tree.list itemconfigure $id -fill $cvsglb(hlfg)
-    $w.tree.list lower $i
-  } 
+     # get the bounding box for the text id
+  set bbox [$w.tree.list bbox $w.tree.list.tx$id]
+  if {[llength $bbox] != 4} {
+    return
+  }
+  set lx [lindex $bbox 0]
+  #set uy [lindex $bbox 1]
+  set ly [lindex $bbox 3]
+  set ly [expr {$ly +1}]
+  set uy [expr {$ly -16}]
+  set i [eval $w.tree.list create rectangle \
+    $lx $ly [winfo width $w.tree] $uy \
+    -fill $cvsglb(hlbg) -tag HBox$w.tree.list.tx$id -outline \"\"]
+  $w.tree.list itemconfigure $w.tree.list.tx$id -fill $cvsglb(hlfg)
+  $w.tree.list lower $i
+  set i [eval $w.labl.list create rectangle \
+    0 $ly [winfo width $w.labl] $uy \
+    -fill $cvsglb(hlbg) -tag HBox$w.tree.list.tx$id -outline \"\"]
+  $w.labl.list itemconfigure $w.labl.list.tx$id -fill $cvsglb(hlfg)
+  $w.labl.list lower $i
 }
