@@ -7,20 +7,28 @@ proc read_svn_dir {dirname} {
   global cmd
 
   gen_log:log T "ENTER ($dirname)"
- # svn info gets the URL
-  set cmd(info) [exec::new "svn info"]
-  set info_lines [$cmd(info)\::output]
-  foreach infoline [split $info_lines "\n"] {
+  # svn info gets the URL
+  # Have to do eval exec because we need the error output
+  set command "svn info"
+  gen_log:log C "$command"
+  set ret [catch {eval "exec $command"} output]
+  if {$ret} {
+    cvsfail $output
+    return 0
+  }
+  foreach infoline [split $output "\n"] {
     if {[string match "URL*" $infoline]} {
-      #gen_log:log D "$infoline"
+      gen_log:log D "$infoline"
       set cvscfg(url) [lrange $infoline 1 end]
     }
   }
-  $cmd(info)\::destroy
-  catch {unset cmd(info)}
+
+  if {! [info exists cvscfg(url)]} {
+    set cvscfg(url) ""
+  }
   if {$cvscfg(url) == ""} {
     cvsfail "Can't get the SVN URL"
-    return
+    return 0
   }
 
   set root ""
@@ -77,11 +85,11 @@ proc read_svn_dir {dirname} {
     set cvscfg(svnroot) $cvscfg(url)
     gen_log:log D "svnroot: $cvscfg(svnroot)"
     set cvsglb(relpath) ""
-    gen_log:log T "LEAVE (1)"
-    return 1
+    gen_log:log T "LEAVE (-1)"
+    return -1
   }
   gen_log:log T "LEAVE (0)"
-  return 0
+  return 1
 }
 
 # Get stuff for main workdir browser
@@ -114,8 +122,14 @@ proc svn_workdir_status {} {
       #             965       687 joe          wc/zig.c
       set wrev [lindex $varcols 0]
       set crev [lindex $varcols 1]
-      set cauthor [lindex $varcols 2]
-      set filename [lrange $varcols 3 end]
+      set nb [string first "/emailAddress=" [lrange $varcols 3 end] ]
+      if {$nb == "-1"} {
+        set cauthor [lindex $varcols 2]
+        set filename [lrange $varcols 3 end]
+      } else {
+        set cauthor [lrange $varcols 2 3]
+        set filename [lrange $varcols 4 end]
+      }
     } else {
       #?                                       newfile
       set filename [lrange $logline 1 end]
@@ -182,34 +196,34 @@ proc svn_add {args} {
 # does svn remove from workdir browser
 proc svn_remove {args} {
   global cvscfg
-  global cmd
 
   gen_log:log T "ENTER ($args)"
   set filelist [join $args]
 
-  set cmd [exec::new "svn remove $filelist"]
-  auto_setup_dir $cmd
+  set command [exec::new "svn remove $filelist"]
+  auto_setup_dir $command
 
   gen_log:log T "LEAVE"
 }
 
 # called from the workdir browser checkmark button
-proc svn_check {directory {v {0}} } {
+proc svn_check {directory} {
   global cvscfg
 
-  gen_log:log T "ENTER ($directory $v)"
+  gen_log:log T "ENTER ($directory)"
 
   busy_start .workdir.main
 
-  set flags ""
-  if {$v} {
-    append flags "uv"
-  }
+  # Always show updates
+  set flags "-u"
+  # Only recurse if flag is set
   if {! $cvscfg(recurse)} {
     append flags "N"
   }
-  if {$flags != ""} {
-    set flags "-$flags"
+  # unknown files are removed by the filter but we might as well minimize
+  # the work the filter has to do
+  if {$cvscfg(status_filter)} {
+    append flags "q"
   }
   set command "svn status $flags $directory"
   set check_cmd [viewer::new "SVN Status Check"]
@@ -331,24 +345,24 @@ proc svn_commit_dialog {} {
   pack .commit.message -in .commit.top -padx 2 -pady 5
 
 
-  button .commit.ok -text "OK" \
+  button .commit.ok -text "OK" -highlightbackground $cvsglb(bg) \
     -command {
       grab release .commit
       wm withdraw .commit
       set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
       svn_commit $cvsglb(commit_comment) $cvsglb(commit_list)
     }
-  button .commit.apply -text "Apply" \
+  button .commit.apply -text "Apply" -highlightbackground $cvsglb(bg) \
     -command {
       set cvsglb(commit_comment) [.commit.tcomment get 1.0 end]
       svn_commit $cvsglb(commit_comment) $cvsglb(commit_list)
     }
-  button .commit.clear -text "ClearAll" \
+  button .commit.clear -text "ClearAll" -highlightbackground $cvsglb(bg) \
     -command {
       set version ""
       .commit.tcomment delete 1.0 end
     }
-  button .commit.quit \
+  button .commit.quit -highlightbackground $cvsglb(bg) \
     -command {
       grab release .commit
       wm withdraw .commit
@@ -427,7 +441,6 @@ proc svn_commit {comment args} {
     setup_dir
   }
   gen_log:log T "LEAVE"
-
 }
 
 # Called from workdir browser annotate button
@@ -586,8 +599,7 @@ proc svn_jit_listdir { tf into } {
 
   busy_start $tf
   ModTree:close $tf /$dir
-  #puts "<- delitem /$dir/d"
-  ModTree:delitem $tf /$dir/d
+  ModTree:delitem $tf /$dir/_jit_placeholder
   foreach f $fils {
     set command "ModTree:newitem $tf \"/$dir/$f\" \"$f\" \"$info($f)\" -image Fileview"
     set r [catch "$command" err]
@@ -605,9 +617,9 @@ proc svn_jit_listdir { tf into } {
 
 proc svn_jit_dircmd { tf dir } {
   global cvscfg
+  global Tree
 
-  gen_log:log T "ENTER ($tf $dir)"
-  #puts "\nEntering svn_jit_dircmd ($dir)"
+  #gen_log:log T "ENTER ($tf $dir)"
 
   # Here we are just figuring out if the top level directory is empty or not.
   # We don't have to collect any other information, so no -v flag
@@ -621,6 +633,7 @@ proc svn_jit_dircmd { tf dir } {
   }
   set lbl "[file tail $dir]/"
   set exp "([llength $contents] items)"
+  set parent "[file root $dir]"
 
   set dirs {}
   set fils {}
@@ -635,20 +648,24 @@ proc svn_jit_dircmd { tf dir } {
     }
   }
 
+  # To avoid having to look ahead and build the whole tree at once, we put
+  # a "marker" item in non-empty directories so it will look non-empty
+  # and be openable
   if {$dirs == {} && $fils == {}} {
-    #puts "  $dir is empty"
     catch "ModTree:newitem $tf \"/$dir\" \"$lbl\" \"$exp\" -image Folder"
   } else {
-    #puts "  $dir has contents"
+    # Newitem returns nothing if the item already exists, or an "after" from
+    # buildwhenidle if the item had to be inserted
     set r [catch "ModTree:newitem $tf \"/$dir\" \"$lbl\" \"$exp\" -image Folder" err]
     if {! $r} {
-      #puts "-> newitem /$dir/d"
-      catch "ModTree:newitem $tf \"/$dir/d\" d d -image {}"
+      if {! $Tree($tf:/$dir:open)} {
+        # If the node is already open, we don't need a placeholder
+        catch "ModTree:newitem $tf \"/$dir/_jit_placeholder\" \"\" \"\" -image {}"
+      }
     }
   }
 
-  gen_log:log T "LEAVE"
-  #puts "Leaving svn_jit_dircmd\n"
+  #gen_log:log T "LEAVE"
 }
 
 # called from module browser - list branches & tags
@@ -697,33 +714,25 @@ proc parse_svnmodules {tf svnroot} {
   gen_log:log T "LEAVE"
 }
 
-proc svn_cat {rev file} {
-  gen_log:log T "ENTER ($rev $file)"
-
-  set cat_cmd [viewer::new "SVN cat $rev $file"]
-  set command "svn -r $rev cat $file"
-  $cat_cmd\::do "$command" 0
-
-  gen_log:log T "LEAVE"
-}
- 
 # called from workdir Reports menu
 proc svn_log {args} {
   global cvscfg
   gen_log:log T "ENTER ($args)"
 
   set filelist [join $args]
-  set command "svn log "
-  if {$cvscfg(ldetail) == "latest"} {
-    append command "-r COMMITTED "
-  }
-  if {$cvscfg(ldetail) == "summary"} {
-    append command "-q "
-  }
-  append command $filelist
+  foreach file $filelist {
+    set command "svn log "
+    if {$cvscfg(ldetail) == "latest"} {
+      append command "-r COMMITTED "
+    }
+    if {$cvscfg(ldetail) == "summary"} {
+      append command "-q "
+    }
+    append command "\"$file\""
 
-  set logcmd [viewer::new "SVN Log ($cvscfg(ldetail))"]
-  $logcmd\::do "$command"
+    set logcmd [viewer::new "SVN Log $file ($cvscfg(ldetail))"]
+    $logcmd\::do "$command"
+  }
   gen_log:log T "LEAVE"
 }
 
@@ -732,12 +741,13 @@ proc svn_merge_conflict {args} {
 
   gen_log:log T "ENTER ($args)"
 
-  set filelist [join $args]
-  if {$filelist == ""} {
-    cvsfail "Please select some files to merge first!"
+  if {[llength $args] != 1} {
+    cvsfail "Please select one file."
     return
   }
+  set filelist [join $args]
 
+  # See if it's really a conflict file
   foreach file $filelist {
     gen_log:log F "OPEN $file"
     set f [open $file]
@@ -753,9 +763,11 @@ proc svn_merge_conflict {args} {
     close $f
    
     if { $match != 1 } { 
-      cvsfail "This file does not appear to have a conflict." .workdir
-      return
+      cvsfail "$file does not appear to have a conflict." .workdir
+      continue
     }
+  # FIXME: we don't want to tie up the whole UI with tkdiff, but
+  # if we don't wait, we have no way to know if we can mark resolved
     # Invoke tkdiff with the proper option for a conflict file
     # and have it write to the original file
     set command "$cvscfg(tkdiff) -conflict -o \"$file\" \"$file\""
@@ -779,6 +791,43 @@ proc svn_merge_conflict {args} {
   gen_log:log T "LEAVE"
 }
 
+proc svn_resolve {args} {
+  global cvscfg
+
+  gen_log:log T "ENTER ($args)"
+  set filelist [join $args]
+
+  # See if it still has a conflict
+  foreach file $filelist {
+    gen_log:log F "OPEN $file"
+    set f [open $file]
+    set match 0
+    while { [eof $f] == 0 } {
+      gets $f line
+      if { [string match "<<<<<<< *" $line] } {
+        set match 1
+        break
+      }
+    }
+    gen_log:log F "CLOSE $file"
+    close $f
+
+    if {$match} {
+      set mess "$file still contains \"<<<<<<< \" - \nUnmark anyway?"
+      if {[cvsalwaysconfirm $mess .workdir] != "ok"} {
+        continue
+      }
+    }
+    gen_log:log D "Marking $file as resolved"
+    set command [exec::new "svn resolved $file"]
+  }
+  if {$cvscfg(auto_status)} {
+    setup_dir
+  }
+
+  gen_log:log T "LEAVE"
+}
+
 proc svn_revert {args} {
   global cvscfg
 
@@ -788,77 +837,50 @@ proc svn_revert {args} {
     set filelist "-R ."
   }
   gen_log:log D "Reverting $filelist"
-  set cmd [exec::new "svn revert $filelist"]
-  auto_setup_dir $cmd
+  set command [exec::new "svn revert $filelist"]
+  auto_setup_dir $command
 
   gen_log:log T "LEAVE"
 }
 
-proc svn_tag {tagname force branch update args} {
+proc svn_tag {tagname b_or_t update args} {
 #
-# This tags a file or directory.
+# This tags a file or directory in the current sandbox.
 #
   global cvscfg
   global cvsglb
 
-  gen_log:log T "ENTER ($tagname $force $branch $update $args)"
-
+  gen_log:log T "ENTER ($tagname $b_or_t $update $args)"
+  
   if {$tagname == ""} {
     cvsfail "You must enter a tag name!" .workdir
     return 1
   }
   set filelist [join $args]
+  gen_log:log D "relpath: $cvsglb(relpath)  filelist \"$filelist\""
 
-  set v [viewer::new "SVN Tag (Copy)"]
+  if {$b_or_t == "tag"} {set pathelem "tags"}
+  if {$b_or_t == "branch"} {set pathelem "branches"}
 
-  # Can't use file join or it will mess up the URL
-  if {$branch == "yes"} {
-    set to_path "$cvscfg(svnroot)/branches/$tagname"
-    set comment "Branched_using_TkSVN"
-  } else {
-    set to_path "$cvscfg(svnroot)/tags/$tagname"
-    set comment "Tagged_using_TkSVN"
-  }
-  set ret [catch "eval exec svn list $to_path" err]
-  if {$ret} {
-    set command "svn mkdir -m\"$comment\" $to_path"
+  set comment "${b_or_t}_copy_by_TkSVN"
+  set v [viewer::new "SVN Copy $tagname"]
+  set to_url "$cvscfg(svnroot)/$pathelem/$tagname/$cvsglb(relpath)"
+  if { $filelist == {} } {
+    set command "svn copy -m\"$comment\" $cvscfg(url) $to_url"
+    $v\::log "$command"
     $v\::do "$command"
-    $v\::wait
-  }
-
-  # We may need to construct a path to copy the file to
-  set cum_path ""
-  set pathelements [file split $cvsglb(relpath)]
-  set depth [llength $pathelements]
-  if {$filelist == ""} {
-    incr depth -1
-  }
-  for {set i 0} {$i < $depth} {incr i} {
-    set cum_path [file join $cum_path [lindex $pathelements $i]]
-    gen_log:log D "  $i $cum_path"
-    set ret [catch "eval exec svn list $to_path/$cum_path" err]
-    if {$ret} {
-      set command "svn mkdir -m\"$comment\" $to_path/$cum_path"
-      $v\::do "$command"
-      $v\::wait
-    }
-  }
-
-  if {$cvsglb(relpath) == "" && $args == "{}" } {
-    set ret [catch "eval exec svn ls" view_this]
-    if {$ret} {
-      cvsfail "$view_this" .workdir
-    } else {
-      set flist [split $view_this "\n"]
-      foreach f $flist {
-        $v\::do "svn copy \"$f\" \"$to_path/$cum_path\" -m\"$comment\""
+  } else {
+    foreach f $filelist {
+      if {$f == "."} {
+        set command "svn copy -m\"comment\" $cvscfg(url) $to_url"
+      } else {
+        svn_pathforcopy $tagname $pathelem $v
+        set from_url [safe_url $cvscfg(url)/$f]
+        set command "svn copy -m\"$comment\" $from_url $to_url"
       }
-      $v\::wait
+      $v\::log "$command"
+      $v\::do "$command"
     }
-  } else {
-    set command "svn copy $filelist -m\"$comment\" $to_path/$cum_path"
-    $v\::do "$command"
-    $v\::wait
   }
 
   if {$update == "yes"} {
@@ -874,28 +896,112 @@ proc svn_tag {tagname force branch update args} {
   gen_log:log T "LEAVE"
 }
 
-proc svn_rcopy {from_path to_path} {
+proc svn_rcopy {from_path b_or_t newtag} {
 #
 # makes a tag or branch.  Called from the module browser
 #
   global cvscfg
   global cvsglb
 
-  gen_log:log T "ENTER ($from_path $to_path)"
+  gen_log:log T "ENTER ($from_path $b_or_t $newtag)"
 
-  set v [viewer::new "SVN Copy"]
-  set command "svn copy [safe_url $from_path]"
-  # Can't use file join or it will mess up the URL
+  # We're going to do some dangerous second-guessing here.  If "trunk", or
+  # something just below the "branches" or "tags" path, is selected, we guess
+  # they want to copy its contents.
+  # Otherwise, we'll copy exactly what they have selected.
+  set need_list 0
+  set idx [string length $cvscfg(svnroot)]
+  incr idx ;# advance past /
+  set from_path_remainder [string range $from_path $idx end]
+  set pathelements [file split $from_path_remainder]
+  if {$from_path_remainder == "trunk"} {
+    set need_list 1
+  } elseif {[llength $pathelements] == 2} {
+    set from_type [lindex $pathelements 0]
+    switch -- $from_type {
+      "tags" -
+      "branches" {
+        set need_list 1
+      }
+    }
+  }
+  set comment "${b_or_t}_copy_by_TkSVN"
+
+  set v [viewer::new "SVN Copy $newtag"]
   set comment "Copied_using_TkSVN"
-  append command " [safe_url $to_path] -m\"$comment\""
-  $v\::do "$command"
-  $v\::wait
+  set to_path [svn_pathforcopy $newtag $b_or_t $v]
 
+  if {! $need_list } {
+    # Copy the selected path
+    set command "svn copy -m\"$comment\" [safe_url $from_path] $to_path"
+    $v\::do "$command"
+    $v\::wait
+  } else {
+    # Copy the contents of the selected path
+    set command "svn list [safe_url $from_path]"
+    set cmd(svnlist) [exec::new "$command"]
+    if {[info exists cmd(svnlist)]} {
+      set contents [split [$cmd(svnlist)\::output] "\n"]
+      $cmd(svnlist)\::destroy
+      catch {unset cmd(svnlist)}
+    }
+    foreach f $contents {
+      if {$f == ""} {continue}
+      set file_from_path [safe_url "$from_path/$f"]
+      set command "svn copy -m\"$comment\" $file_from_path $to_path"
+      $v\::log "$command\n"
+      $v\::do "$command"
+      $v\::wait
+    }
+  }
+  # Update with what we've done
   modbrowse_run svn
   gen_log:log T "LEAVE"
 }
 
-proc svn_merge {fromrev sincerev frombranch mtag ftag url} {
+proc svn_pathforcopy {tagname b_or_t viewer} {
+# For svn copy, the destination path in the repository must already
+# exist. If we're tagging somewhere other than the top level, it may
+# not exist yet.  This proc creates the path if necessary and returns
+# it to the calling proc, which will do the copy.
+  global cvscfg
+  global cvsglb
+
+  gen_log:log T "ENTER (\"$tagname\" \"$b_or_t\" \"$viewer\")"
+  # Can't use file join or it will mess up the URL
+  set to_path [safe_url "$cvscfg(svnroot)/$b_or_t/$tagname"]
+  set comment "${b_or_t}_directory_path_by_TkSVN"
+
+  # If no file yet has this tag/branch name, create it
+  set ret [catch "eval exec svn list $to_path" err]
+  if {$ret} {
+    set command "svn mkdir -m\"$comment\" $to_path"
+    $viewer\::log "$command\n"
+    $viewer\::do "$command"
+    $viewer\::wait
+  }
+  # We may need to construct a path to copy the file to
+  set cum_path ""
+  set pathelements [file split $cvsglb(relpath)]
+  set depth [llength $pathelements]
+  for {set i 0} {$i < $depth} {incr i} {
+    set cum_path [file join $cum_path [lindex $pathelements $i]]
+    gen_log:log D "  $i $cum_path"
+    set ret [catch "eval exec svn list $to_path/$cum_path" err]
+    if {$ret} {
+      set command "svn mkdir -m\"$comment\" $to_path/$cum_path"
+      $viewer\::do "$command"
+      $viewer\::wait
+    }
+  }
+  if {$cum_path != ""} {
+    set to_path "$to_path/$cum_path"
+  }
+  gen_log:log T "LEAVE (\"$to_path\")"
+  return $to_path
+}
+
+proc svn_merge {parent frompath since currentpath frombranch args} {
 #
 # This does a join (merge) of a chosen revision of localfile to the
 # current revision.
@@ -903,80 +1009,96 @@ proc svn_merge {fromrev sincerev frombranch mtag ftag url} {
   global cvscfg
   global cvsglb
 
-  gen_log:log T "ENTER ($fromrev $sincerev $frombranch $mtag $ftag $url)"
+  gen_log:log T "ENTER( \"$frompath\" \"$since\" \"$currentpath\" \"$frombranch\" $args)"
 
-  set v [viewer::new "SVN Merge"]
+  set mergetags [assemble_mergetags $frombranch]
+  set curr_tag [lindex $mergetags 0]
+  set fromtag [lindex $mergetags 1]
+  set totag [lindex $mergetags 2]
 
-  if {$cvscfg(auto_tag)} {
-    set tagpath $cvscfg(svnroot)/tags/$mtag
-    set comment "Tagged_using_TkSVN"
-    set ret [catch "eval exec svn list $tagpath" err]
-    if {$ret} {
-      set command "svn mkdir -m\"$comment\" $tagpath"
-      $v\::do "$command"
-      $v\::wait
-    }
-    # We may need to construct a path to copy the file to
-    set cum_path ""
-    set pathelements [file split $cvsglb(relpath)]
-    set depth [llength $pathelements]
-    incr depth -1
-    for {set i 0} {$i < $depth} {incr i} {
-      set cum_path [file join $cum_path [lindex $pathelements $i]]
-      gen_log:log D "  $i $cum_path"
-      set ret [catch "eval exec svn list $tagpath/$cum_path" err]
-      if {$ret} {
-        set command "svn mkdir -m\"$comment\" $tagpath/$cum_path"
-        $v\::do "$command"
-        $v\::wait
-      }
-    }
-
-    set comment "Copied_using_TkSVN"
-    if {$cvsglb(relpath) == "" && [string range $url end-1 end] == "/."} {
-      set ret [catch "eval exec svn ls" view_this]
-      if {$ret} {
-        cvsfail "$view_this" .
-      } else {
-        set flist [split $view_this "\n"]
-        set trimurl [string range $url 0 end-2]
-        foreach f $flist {
-          $v\::do "svn copy \"$trimurl/$f\" \"$tagpath/$cum_path\" -m\"$comment\""
-        }
-        $v\::wait
-      }
-    } else {
-      $v\::do "svn copy \"$url\" $tagpath/$cum_path -m\"$comment\""
-      $v\::wait
-    }
-
-    toplevel .reminder
-    message .reminder.m1 -aspect 600 -text \
-      "When you are finished checking in your merges, \
-      you should apply the tag"
-    entry .reminder.ent -width 32 -relief groove \
-       -font $cvscfg(guifont) -readonlybackground $cvsglb(readonlybg)
-    .reminder.ent insert end $ftag 
-    .reminder.ent configure -state readonly
-    message .reminder.m2 -aspect 600 -text \
-      "using the \"Tag the selected files\" button"
-    frame .reminder.bottom -relief raised -bd 2
-    button .reminder.bottom.close -text "Dismiss" \
-      -command {destroy .reminder}
-    pack .reminder.bottom -side bottom -fill x
-    pack .reminder.bottom.close -side bottom -expand yes
-    pack .reminder.m1 -side top
-    pack .reminder.ent -side top -padx 2
-    pack .reminder.m2 -side top
+  regsub {^.*@} $frompath {r} from
+  if {$since == {}} {
+    set mess "Merge revision $from\n"
+  } else {
+    set mess "Merge the changes between revision\n $since and $from"
+    append mess " (if $since > $from the changes are removed)\n"
+  }
+  append mess " to the current revision ($curr_tag)"
+  if {[cvsalwaysconfirm $mess $parent] != "ok"} {
+    return
   }
 
-  set fromrev [string trimleft $fromrev {r}]
-  set sincerev [string trimleft $sincerev {r}]
-  set command "svn merge -r$sincerev\:$fromrev $url"
-    
-  $v\::do "$command" 0 status_colortags
+  # Do the update here, and defer the tagging until later
+  set commandline "svn merge \"$currentpath\" \"$frompath\""
+  set v [viewer::new "SVN Merge"]
+  $v\::do "$commandline" 1 status_colortags
   $v\::wait
 
+  if [winfo exists .workdir] {
+    if {$cvscfg(auto_status)} {
+      setup_dir
+    }
+  } else {
+    workdir_setup
+  }
+
+  dialog_merge_notice svn $from $frombranch $fromtag $totag $args
+
+  gen_log:log T "LEAVE"
+}
+
+proc svn_merge_tag_seq {from frombranch totag fromtag args} {
+  global cvscfg
+  global cvsglb
+
+  gen_log:log T "ENTER (\"$from\" \"$totag\" \"$fromtag\" $args)"
+
+  set filelist ""
+  foreach f $args {
+    append filelist "\"$f\" "
+  }
+
+  # It's muy importante to make sure everything is OK at this point
+  set commandline "svn status -uq $filelist"
+  gen_log:log C "$commandline"
+  set ret [catch {eval "exec $commandline"} view_this]
+  set logmode [expr {$ret ? {E} : {D}}]
+  view_output::new "SVN Check" $view_this
+  gen_log:log $logmode $view_this
+  if {$ret} {
+    set mess "SVN Check shows errors which would prevent a successful\
+    commit. Please resolve them before continuing."
+    if {[cvsalwaysconfirm $mess .workdir] != "ok"} {
+      return
+    }
+  }
+
+  # Do the commit
+  set v [viewer::new "SVN Commit a Merge"]
+  $v\::log "svn commit -m \"Merge from $from\" $filelist\n"
+  $v\::do "svn commit -m \"Merge from $from\" $filelist" 1
+  $v\::wait
+
+  # Tag if desired (no means not a branch
+  if {$cvscfg(auto_tag) && $fromtag != ""} {
+    if {$frombranch == "trunk"} {
+      set from_path "$cvscfg(svnroot)/trunk/$cvsglb(relpath)"
+    } else {
+      set from_path "$cvscfg(svnroot)/branches/$frombranch/$cvsglb(relpath)"
+    }
+    # tag the current (mergedto) branch
+    svn_tag $fromtag tags false $args ;# opens its own viewer
+    # Tag the mergedfrom branch
+    set filelist [join $args]
+    foreach file $filelist {
+      if {$file == "."} {
+        svn_rcopy [safe_url $from_path] "tags" $totag ;# opens its own viewer
+      } else {
+        svn_rcopy [safe_url $from_path/$f] "tags" $totag ;# opens its own viewer
+      }
+    }
+  }
+
   if {$cvscfg(auto_status)} {
     setup_dir
   }
@@ -1051,16 +1173,16 @@ proc svn_fileview {revision filename kind} {
   global cvscfg
 
   gen_log:log T "ENTER ($revision $filename $kind)"
-  set cmd "cat"
+  set command "cat"
   if {$kind == "directory"} {
-     set cmd "ls"
+     set command "ls"
   }
   if {$revision == {}} {
-    set command "svn $cmd \"$filename\""
+    set command "svn $command \"$filename\""
     set v [viewer::new "$filename"]
     $v\::do "$command"
   } else {
-    set command "svn $cmd -$revision \"$filename\""
+    set command "svn $command -$revision \"$filename\""
     set v [viewer::new "$filename Revision $revision"]
     $v\::do "$command"
   }
@@ -1103,7 +1225,13 @@ proc svn_branches {files} {
 }
 
 proc safe_url { url } {
+  # Replacement is done in an ordered manner, so the  key  appearing
+  # first  in  the list will be checked first, and so on.  string is
+  # only iterated over once.
   set url [string map {
+    "%20" "%20"
+    "%25" "%25"
+    "%26" "%26"
     "%" "%25"
     "&" "%26"
     " " "%20"
@@ -1225,7 +1353,7 @@ namespace eval ::svn_branchlog {
         # Can't use file join or it will mess up the URL
         set safe_filename [safe_url $filename]
         set path "$cvscfg(url)/$safe_filename"
-        $ln\::ConfigureButtons $path
+        $ln\::ConfigureButtons $filename
 
         # Find out where to put the working revision icon (if anywhere)
         set command "svn log -q --stop-on-copy \"$filename\""
@@ -1242,7 +1370,7 @@ namespace eval ::svn_branchlog {
         } else {
           set path "$cvscfg(svnroot)/trunk/$relpath/$safe_filename"
         }
-        if {[read_svn_dir .] == 1} {
+        if {[read_svn_dir .] == -1} {
           set path "$cvscfg(svnroot)/$safe_filename"
           if {! [info exists cvscfg(svnconform_seen)]} {
             set msg "Your repository does not seem to be arranged in trunk, branch, and root directories.  The Branch Browser can't detect branches and tags."
@@ -1304,6 +1432,9 @@ namespace eval ::svn_branchlog {
         set branches [$cmd_log\::output]
         $cmd_log\::destroy
 
+        # There can be files such as "README" here that aren't branches
+        set branches [grep_filter {/$} $branches]
+
         if {[info exists cvscfg(svn_branch_filter)] && \
             [info exists cvscfg(svn_branch_max_count)]} {
           # Only include branches that match regexp svn_branch_filter.
@@ -1316,7 +1447,7 @@ namespace eval ::svn_branchlog {
         foreach branch $branches {
           gen_log:log D "$branch"
           # There can be files such as "README" here that aren't branches
-          if {![string match {*/} $branch]} {continue}
+          #if {![string match {*/} $branch]} {continue}
           set branch [string trimright $branch "/"]
           # Can't use file join or it will mess up the URL
           gen_log:log D "BRANCHES: RELPATH \"$relpath\""
@@ -1334,6 +1465,7 @@ namespace eval ::svn_branchlog {
           }
           set loglines [split $log_output "\n"]
           set rb [parse_svnlog $loglines $branch]
+          #puts "parse_svnlog returned base revision $rb for $branch"
           # See if the current revision is on this branch
           set curr 0
           set brevs $branchrevs($branch)
@@ -1377,9 +1509,19 @@ namespace eval ::svn_branchlog {
           # back one, before You are Here
           set idx [llength $branchrevs($branch)]
           if {$curr} {
+            #puts "Currently at Top: decrementing branchpoint"
+            gen_log:log E "Currently at Top: decrementing branchpoint"
             incr idx -1
           }
           set bp [lindex $allrevs($branch) $idx]
+          if {$bp == ""} {
+            #puts "Allrevs same as branchrevs: decrementing branchpoint"
+            gen_log:log E "Allrevs same as branchrevs: decrementing branchpoint"
+            set bp [lindex $branchrevs($branch) end]
+            set bpn [string trimleft $bp "r"]
+            incr bpn -1
+            set bp "r${bpn}"
+          }
           lappend revbranches($bp) $rb
         }
         # Tags
