@@ -64,7 +64,7 @@ set cvscfg(bitmapdir) [file join $TclRoot tkcvs bitmaps]
 #puts "TCDIR $TCDIR"
 #puts "BITMAPDIR $cvscfg(bitmapdir)"
 
-set cvscfg(version) "8.0.4"
+set cvscfg(version) "8.1"
 
 if {! [info exists cvscfg(editorargs)]} {
   set cvscfg(editorargs) {}
@@ -126,6 +126,8 @@ if {! [get_cde_params]} {
   set cvsglb(readonlybg) gray96
   set cvsglb(hlbg) [lindex [.testent configure -selectbackground] 4]
   set cvsglb(hlfg) [lindex [.testent configure -selectforeground] 4]
+  set hlbg_rgb [winfo rgb .testent $cvsglb(hlbg)]
+  set cvscfg(listboxfont) [lindex [.testent configure -font] 4]
   destroy .testent
 
 
@@ -145,22 +147,33 @@ if {! [get_cde_params]} {
   # backgrounds the same which I don't like, so I'm going to use the same
   # trick I use for CDE to give the canvases a little shading.  I don't
   # do this for raw X11 because the user might have set their own options.
-  set bg [option get .testlbl background background]
-  if {[string length $bg]} {
-    set rgb_bg [winfo rgb .testlbl $bg]
-    set shadow [format #%02x%02x%02x [expr (9*[lindex $rgb_bg 0])/2560] \
-                                     [expr (9*[lindex $rgb_bg 1])/2560] \
-                                     [expr (9*[lindex $rgb_bg 2])/2560]]
-    set cvsglb(canvbg) $shadow
-  }
+  set cvsglb(bg) [lindex [.testlbl cget -background] 0]
+  set cvsglb(fg) [lindex [.testlbl cget -foreground] 0]
+  set cvslb(dfg) [lindex [.testlbl cget -disabledforeground] 0]
+  set cvsglb(canvbg) [rgb_shadow $cvsglb(bg)]
   destroy .testlbl
-   
-  # Find out what the default font is for listboxes
-  if { ! [info exists cvscfg(listboxfont)] } {
-    entry .testent
-    set cvscfg(listboxfont) [lindex [.testent configure -font] 4]
-    destroy .testent
+
+  #puts "bg $cvsglb(bg)"
+  #puts "canvbg $cvsglb(canvbg)"
+  #puts "hlbg $cvsglb(hlbg)"
+  if {[is_gray $cvsglb(hlbg)]} {
+    # Which is better? I like the yellow
+    #set cvsglb(hlbg) $cvsglb(sel)
+    set sel [option get . selectColor Background]
+    if {$sel != ""} {
+      #puts "option selectColor: $sel"
+      set  cvsglb(hlbg) $sel
+    } else {
+      set cvsglb(hlbg) "#ffec8b"
+    }
+    #puts "Changed hlbg so it won't be gray"
   }
+  if {[rgb_diff $cvsglb(hlbg) $cvsglb(canvbg)] < 12000} {
+    set cvsglb(hlbg) [rgb_shadow  $cvsglb(hlbg)]
+    #puts "Changed hlbg because it's nearly the same as the canvas ($rgb_diff < 12000)"
+  }
+  #puts "hlbg $cvsglb(hlbg)"
+   
   scrollbar .scrl
   destroy .scrl
 
@@ -174,8 +187,35 @@ if {! [get_cde_params]} {
   option add *Menu.font $menufont userDefault
   option add *Scrollbar.troughColor $cvsglb(canvbg) userDefault
   option add *Canvas.Background $cvsglb(canvbg) userDefault
+} else {
+  set lbf [font actual $cvscfg(listboxfont)]
+  set ffam [lindex $lbf 1]
+  set fsiz [lindex $lbf 3]
+  set cvscfg(listboxfont) [list $ffam $fsiz]
 }
 
+set cvscfg(flashfont) $cvscfg(listboxfont)
+set fsiz [lindex $cvscfg(listboxfont) 1]
+set lbf [font actual $cvscfg(listboxfont)]
+#puts "actual listboxfont: $lbf"
+set ffam [lindex $lbf 1]
+#set fsiz [lindex $lbf 3]
+if {$fsiz == ""} {
+  # This happens on Windows in tk8.5
+  set fsiz [lindex $lbf 3]
+}
+
+set cvscfg(flashfont) [list $ffam $fsiz underline]
+#puts "\nflashfont: $cvscfg(flashfont)"
+#puts "actual flashfont: [font actual $cvscfg(flashfont)]"
+# Prefer underline, but it isn't working at all in tk8.5 on linux
+if {! [font actual $cvscfg(flashfont) -underline]} {
+  puts "Underline font not working.  Try $ffam $fsiz bold"
+  set cvscfg(flashfont) [list [lindex $lbf 1] $fsiz bold]
+  #set cvscfg(flashfont) [list [lindex $lbf 1] [lindex $lbf 3] bold]
+}
+#puts "final flashfont: $cvscfg(flashfont)"
+
 option add *ToolTip.background  "LightGoldenrod1" userDefault
 option add *ToolTip.foreground  "black" userDefault
 
