@@ -58,7 +58,7 @@ proc workdir_setup {} {
   ::picklist::bind .workdir.top.tcwd <Return> \
      {if {[pwd] != $cwd} {change_dir "$cwd"}}
 
-  button .workdir.top.updir_btn -image updir \
+  button .workdir.top.updir_btn -image updir -highlightbackground $cvsglb(bg) \
     -command {change_dir ..}
 
   label .workdir.top.lmodule -text "Path"
@@ -69,12 +69,12 @@ proc workdir_setup {} {
      -anchor w -relief groove
 
   # Make the Repository Browser button prominent
-  button .workdir.top.bmodbrowse -image Modules \
+  button .workdir.top.bmodbrowse -image Modules -highlightbackground $cvsglb(bg) \
      -command modbrowse_run
 
   label .workdir.top.lcvsroot -text "CVSROOT"
   entry .workdir.top.tcvsroot -textvariable cvscfg(cvsroot) \
-     -relief groove -state readonly \
+     -relief groove -bd 2 -state readonly \
      -font $cvscfg(guifont) -readonlybackground $cvsglb(readonlybg)
 
   grid columnconf .workdir.top 1 -weight 1
@@ -107,7 +107,7 @@ proc workdir_setup {} {
      -textvariable cvsglb(default_ignore_filter)
   label .workdir.bottom.filters.space -text "    "
   button .workdir.bottom.filters.cleanbutton -text "Clean:" \
-     -pady 0 -highlightthickness 0 \
+     -pady 0 -highlightthickness 0 -highlightbackground $cvsglb(bg) \
      -command workdir_cleanup
   entry .workdir.bottom.filters.cleanentry -width 12 \
      -textvariable cvscfg(clean_these)
@@ -140,55 +140,55 @@ proc workdir_setup {} {
   #
   # Action buttons along the bottom of the screen.
   #
-  button .workdir.bottom.buttons.funcs.bedit_files -image Fileedit \
+  button .workdir.bottom.buttons.funcs.bedit_files -image Fileedit -highlightbackground $cvsglb(bg) \
      -command { workdir_edit_file [workdir_list_files] }
-  button .workdir.bottom.buttons.funcs.bview_files -image Fileview \
+  button .workdir.bottom.buttons.funcs.bview_files -image Fileview -highlightbackground $cvsglb(bg) \
      -command { workdir_view_file [workdir_list_files] }
-  button .workdir.bottom.buttons.funcs.bdelete_file -image Delete \
+  button .workdir.bottom.buttons.funcs.bdelete_file -image Delete -highlightbackground $cvsglb(bg) \
      -command { workdir_delete_file [workdir_list_files] }
-  button .workdir.bottom.buttons.funcs.bmkdir -image Dir_new \
+  button .workdir.bottom.buttons.funcs.bmkdir -image Dir_new -highlightbackground $cvsglb(bg) \
      -command { file_input_and_do "New Directory" workdir_newdir}
 
-  button .workdir.bottom.buttons.dirfuncs.brefresh -image Refresh \
+  button .workdir.bottom.buttons.dirfuncs.brefresh -image Refresh -highlightbackground $cvsglb(bg) \
      -command { change_dir [pwd] }
   button .workdir.bottom.buttons.dirfuncs.bcheckdir -image Check
 
-  button .workdir.bottom.buttons.cvsfuncs.blogfile -image Branches \
+  button .workdir.bottom.buttons.cvsfuncs.blogfile -image Branches -highlightbackground $cvsglb(bg) \
      -command { cvs_branches [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bannotate -image Annotate \
+  button .workdir.bottom.buttons.cvsfuncs.bannotate -image Annotate -highlightbackground $cvsglb(bg) \
      -command { cvs_annotate $current_tagname [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bfilelog -image Log \
+  button .workdir.bottom.buttons.cvsfuncs.bfilelog -image Log -highlightbackground $cvsglb(bg) \
     -command { cvs_log [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bdiff -image Diff \
+  button .workdir.bottom.buttons.cvsfuncs.bdiff -image Diff -highlightbackground $cvsglb(bg) \
      -command { comparediff [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bconflict -image Conflict \
+  button .workdir.bottom.buttons.cvsfuncs.bconflict -image Conflict -highlightbackground $cvsglb(bg) \
      -command { cvs_merge_conflict [workdir_list_files] }
 
-  button .workdir.bottom.buttons.cvsfuncs.btag -image Tag \
-     -command { file_tag_dialog "no" }
-  button .workdir.bottom.buttons.cvsfuncs.bbranchtag -image Branchtag \
-     -command { file_tag_dialog "yes" }
-  button .workdir.bottom.buttons.cvsfuncs.badd_files -image Add \
+  button .workdir.bottom.buttons.cvsfuncs.btag -image Tag -highlightbackground $cvsglb(bg) \
+     -command { file_tag_dialog "tag" }
+  button .workdir.bottom.buttons.cvsfuncs.bbranchtag -image Branchtag -highlightbackground $cvsglb(bg) \
+     -command { file_tag_dialog "branch" }
+  button .workdir.bottom.buttons.cvsfuncs.badd_files -image Add -highlightbackground $cvsglb(bg) \
      -command { add_dialog [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bremove -image Remove \
+  button .workdir.bottom.buttons.cvsfuncs.bremove -image Remove -highlightbackground $cvsglb(bg) \
      -command { subtract_dialog [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bcheckin -image Checkin \
+  button .workdir.bottom.buttons.cvsfuncs.bcheckin -image Checkin -highlightbackground $cvsglb(bg) \
       -command cvs_commit_dialog
-  button .workdir.bottom.buttons.cvsfuncs.bupdate -image Checkout
-  button .workdir.bottom.buttons.cvsfuncs.bupdateopts -image CheckoutOpts \
-     -command { update_run }
-  button .workdir.bottom.buttons.cvsfuncs.brevert -image Revert \
+  button .workdir.bottom.buttons.cvsfuncs.bupdate -image Checkout -highlightbackground $cvsglb(bg)
+  button .workdir.bottom.buttons.cvsfuncs.bupdateopts -image CheckoutOpts -highlightbackground $cvsglb(bg) \
+     -command { cvs_update_options }
+  button .workdir.bottom.buttons.cvsfuncs.brevert -image Revert -highlightbackground $cvsglb(bg) \
      -command { cvs_revert [workdir_list_files] }
-  button .workdir.bottom.buttons.cvsfuncs.bjoin -image DirBranches \
+  button .workdir.bottom.buttons.cvsfuncs.bjoin -image DirBranches -highlightbackground $cvsglb(bg) \
      -command cvs_joincanvas
 
-  button .workdir.bottom.buttons.oddfuncs.bcvsedit_files -image Edit \
+  button .workdir.bottom.buttons.oddfuncs.bcvsedit_files -image Edit -highlightbackground $cvsglb(bg) \
      -command { cvs_edit [workdir_list_files] }
-  button .workdir.bottom.buttons.oddfuncs.bunedit_files -image Unedit \
+  button .workdir.bottom.buttons.oddfuncs.bunedit_files -image Unedit -highlightbackground $cvsglb(bg) \
      -command { cvs_unedit [workdir_list_files] }
-  button .workdir.bottom.buttons.oddfuncs.block -image Lock
-  button .workdir.bottom.buttons.oddfuncs.bunlock -image UnLock
-  button .workdir.close -text "Close" \
+  button .workdir.bottom.buttons.oddfuncs.block -image Lock -highlightbackground $cvsglb(bg)
+  button .workdir.bottom.buttons.oddfuncs.bunlock -image UnLock -highlightbackground $cvsglb(bg)
+  button .workdir.close -text "Close" -highlightbackground $cvsglb(bg) \
       -command {
         global cvscfg
         set cvscfg(workgeom) [wm geometry .workdir]
@@ -310,6 +310,7 @@ proc workdir_setup {} {
 
   frame .workdir.main
   pack .workdir.main -side bottom -fill both -expand 1 -fill both
+  update idletasks
 
   if {! [winfo ismapped .workdir]} {
     wm deiconify .workdir
@@ -391,6 +392,7 @@ proc workdir_images {} {
 
 proc workdir_menus {} {
   global cvscfg
+  global cvsglb
   global cvsmenu
   global usermenu
   global execmenu
@@ -458,9 +460,9 @@ proc workdir_menus {} {
   # CVS
   .workdir.menubar.cvs add command -label "Update" -underline 0 \
      -command { \
-        cvs_update {BASE} {Normal} {Remove} {No} { } [workdir_list_files] }
+        cvs_update {BASE} {Normal} {Remove} {recurse} {prune} {No} { } [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Update with Options" -underline 7 \
-     -command update_run
+     -command cvs_update_options
   .workdir.menubar.cvs add command -label "Commit/Checkin" -underline 0 \
      -command cvs_commit_dialog
   .workdir.menubar.cvs add command -label "Add Files" -underline 0 \
@@ -480,7 +482,7 @@ proc workdir_menus {} {
   .workdir.menubar.cvs add command -label "Unset Edit Flag (Unedit)" -underline 11 \
      -command { cvs_unedit [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Tag Files" -underline 0 \
-     -command { file_tag_dialog "no" }
+     -command { file_tag_dialog "tag" }
   .workdir.menubar.cvs add command -label "Browse the Log Diagram" \
      -command { cvs_branches [workdir_list_files] }
   .workdir.menubar.cvs add command -label "Resolve Conflicts" \
@@ -496,6 +498,8 @@ proc workdir_menus {} {
   # SVN
   .workdir.menubar.svn add command -label "Update" -underline 0 \
      -command {svn_update [workdir_list_files]}
+  .workdir.menubar.svn add command -label "Resolve (Un-mark Conflict)" -underline 0 \
+     -command {svn_resolve [workdir_list_files]}
   .workdir.menubar.svn add command -label "Commit/Checkin" -underline 0 \
      -command svn_commit_dialog
   .workdir.menubar.svn add command -label "Add Files" -underline 0 \
@@ -523,51 +527,43 @@ proc workdir_menus {} {
   .workdir.menubar.reports add command -label "Log" -underline 0
   .workdir.menubar.reports add command -label "Annotate/Blame" -underline 0
 
-  set selcolor [option get .workdir selectColor selectColor]
   .workdir.menubar.options add checkbutton -label "Show hidden files" \
      -variable cvscfg(allfiles) -onvalue true -offvalue false \
-     -selectcolor $selcolor -command setup_dir
+     -command setup_dir
   .workdir.menubar.options add checkbutton -label "Automatic directory status" \
-     -variable cvscfg(auto_status) -onvalue true -offvalue false \
-     -selectcolor $selcolor
+     -variable cvscfg(auto_status) -onvalue true -offvalue false
   .workdir.menubar.options add checkbutton -label "Confirmation Dialogs" \
-     -variable cvscfg(confirm_prompt) -onvalue true -offvalue false \
-     -selectcolor $selcolor
+     -variable cvscfg(confirm_prompt) -onvalue true -offvalue false
   .workdir.menubar.options add separator
   .workdir.menubar.options add checkbutton -label "Editor/Author/Locker Column" \
      -variable cvscfg(showeditcol) -onvalue true -offvalue false \
-     -selectcolor $selcolor \
      -command { if {($incvs || $insvn || $inrcs) && $cvscfg(showeditcol)} {
                   DirCanvas:map_column .workdir.main editcol
                 } else {
-                  pack forget .workdir.main.editcol
+                  DirCanvas:unmap_column .workdir.main editcol
                 }
               }
   .workdir.menubar.options add checkbutton -label "Status Column" \
      -variable cvscfg(showstatcol) -onvalue true -offvalue false \
-     -selectcolor $selcolor \
      -command { if {($incvs || $insvn || $inrcs) && $cvscfg(showstatcol)} {
                   DirCanvas:map_column .workdir.main statcol
                 } else {
-                  pack forget .workdir.main.statcol
+                  DirCanvas:unmap_column .workdir.main statcol
                 }
               }
   .workdir.menubar.options add checkbutton -label "Date Column" \
      -variable cvscfg(showdatecol) -onvalue true -offvalue false \
-     -selectcolor $selcolor \
      -command { if {$cvscfg(showdatecol)} {
                   DirCanvas:map_column .workdir.main datecol
                 } else {
-                  pack forget .workdir.main.datecol
+                  DirCanvas:unmap_column .workdir.main datecol
                 }
               }
   .workdir.menubar.options add separator
   .workdir.menubar.options add checkbutton -label "Report->Check Shows Unknown Files" \
-     -variable cvscfg(status_filter) -onvalue false -offvalue true \
-     -selectcolor $selcolor
+     -variable cvscfg(status_filter) -onvalue false -offvalue true
   .workdir.menubar.options add checkbutton -label "Report->Check/Status are Recursive" \
-     -variable cvscfg(recurse) -onvalue true -offvalue false \
-     -selectcolor $selcolor
+     -variable cvscfg(recurse) -onvalue true -offvalue false
   .workdir.menubar.options add cascade -label "CVS Status Detail" \
      -menu .workdir.menubar.options.report_detail
   .workdir.menubar.options add cascade -label "CVS Log Detail" \
@@ -575,7 +571,7 @@ proc workdir_menus {} {
   .workdir.menubar.options add separator
   .workdir.menubar.options add checkbutton -label "Tracing On/Off" \
      -variable cvscfg(logging) -onvalue true -offvalue false \
-     -selectcolor $selcolor -command log_toggle
+     -command log_toggle
   .workdir.menubar.options add cascade -label "Trace Level" \
      -menu .workdir.menubar.options.loglevel
   .workdir.menubar.options add separator
@@ -585,35 +581,35 @@ proc workdir_menus {} {
   menu .workdir.menubar.options.loglevel
   .workdir.menubar.options.loglevel add checkbutton -label "CVS commands (C)" \
      -variable logclass(C) -onvalue "C" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .workdir.menubar.options.loglevel add checkbutton -label "CVS stderr (E)" \
      -variable logclass(E) -onvalue "E" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .workdir.menubar.options.loglevel add checkbutton -label "File creation/deletion (F)"\
      -variable logclass(F) -onvalue "F" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .workdir.menubar.options.loglevel add checkbutton -label "Function entry/exit (T)" \
      -variable logclass(T) -onvalue "T" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
   .workdir.menubar.options.loglevel add checkbutton -label "Debugging (D)" \
      -variable logclass(D) -onvalue "D" -offvalue "" \
-     -selectcolor $selcolor -command gen_log:changeclass
+     -command gen_log:changeclass
 
   menu .workdir.menubar.options.report_detail
   .workdir.menubar.options.report_detail add radiobutton -label "Verbose" \
-     -variable cvscfg(rdetail) -value "verbose" -selectcolor $selcolor
+     -variable cvscfg(rdetail) -value "verbose"
   .workdir.menubar.options.report_detail add radiobutton -label "Summary" \
-     -variable cvscfg(rdetail) -value "summary" -selectcolor $selcolor
+     -variable cvscfg(rdetail) -value "summary"
   .workdir.menubar.options.report_detail add radiobutton -label "Terse" \
-     -variable cvscfg(rdetail) -value "terse" -selectcolor $selcolor
+     -variable cvscfg(rdetail) -value "terse"
 
   menu .workdir.menubar.options.logfile_detail
   .workdir.menubar.options.logfile_detail add radiobutton -label "Summary" \
-     -variable cvscfg(ldetail) -value "summary" -selectcolor $selcolor
+     -variable cvscfg(ldetail) -value "summary"
   .workdir.menubar.options.logfile_detail add radiobutton -label "Latest" \
-     -variable cvscfg(ldetail) -value "latest" -selectcolor $selcolor
+     -variable cvscfg(ldetail) -value "latest"
   .workdir.menubar.options.logfile_detail add radiobutton -label "Verbose" \
-     -variable cvscfg(ldetail) -value "verbose" -selectcolor $selcolor
+     -variable cvscfg(ldetail) -value "verbose"
 
   .workdir.menubar.goto add command -label "Go Home" \
      -command {change_dir $cvscfg(home)}
@@ -799,6 +795,7 @@ proc add_bookmark { } {
 # A listbox to choose a bookmark to delete
 proc delete_bookmark_dialog { } {
    global cvscfg
+   global cvsglb
    global bookmarks
 
    gen_log:log T "ENTER"
@@ -823,10 +820,10 @@ proc delete_bookmark_dialog { } {
    }
    frame $wname.buttons
    pack $wname.buttons -side top -fill x
-   button $wname.delete -text "Delete" \
+   button $wname.delete -text "Delete" -highlightbackground $cvsglb(bg) \
      -command "delete_bookmark $wname"
         
-   button $wname.close -text "Done" \
+   button $wname.close -text "Done" -highlightbackground $cvsglb(bg) \
      -command "
        grab release $wname
        destroy $wname
@@ -1001,7 +998,7 @@ proc setup_dir { } {
     .workdir.bottom.buttons.cvsfuncs.bannotate configure -state normal \
       -command { svn_annotate BASE [workdir_list_files] }
     .workdir.bottom.buttons.cvsfuncs.bconflict configure -state normal \
-      -command { svn_merge_conflict [workdir_list_files] }
+      -command { foreach f [workdir_list_files] {svn_merge_conflict \"$f\"} }
     .workdir.bottom.buttons.cvsfuncs.badd_files configure -state normal
     .workdir.bottom.buttons.cvsfuncs.bremove configure -state normal
     .workdir.bottom.buttons.cvsfuncs.bupdate configure -state normal \
@@ -1021,10 +1018,10 @@ proc setup_dir { } {
     # Reports Menu
     # Check Directory (svn status)
     .workdir.menubar.reports entryconfigure 0 -state normal \
-       -command { svn_check {} 0 }
+       -command { svn_check {} }
     # Status (svn status <filelist>)
     .workdir.menubar.reports entryconfigure 1 -state normal \
-       -command { svn_check [workdir_list_files] 1}
+       -command { svn_check [workdir_list_files] }
     # Log (svn log)
     .workdir.menubar.reports entryconfigure 2 -state normal \
        -command { svn_log [workdir_list_files] }
@@ -1055,9 +1052,9 @@ proc setup_dir { } {
     .workdir.bottom.buttons.cvsfuncs.bremove configure -state normal
     .workdir.bottom.buttons.cvsfuncs.bupdate configure -state normal \
        -command { \
-       cvs_update {BASE} {Normal} {Remove} {No} { } [workdir_list_files] }
+       cvs_update {BASE} {Normal} {Remove} {recurse} {prune} {No} { } [workdir_list_files] }
     .workdir.bottom.buttons.cvsfuncs.bupdateopts configure -state normal \
-     -command { update_run }
+     -command { cvs_update_options }
     .workdir.bottom.buttons.cvsfuncs.bcheckin configure -state normal \
       -command cvs_commit_dialog
     .workdir.bottom.buttons.cvsfuncs.brevert configure -state normal \
@@ -1097,9 +1094,8 @@ proc setup_dir { } {
        -command { cvs_annotate $current_tagname [workdir_list_files] }
   }
 
-  DirCanvas:create .workdir.main \
-    -relief flat -bd 0 -highlightthickness 0 \
-    -width 100 -height 350
+  DirCanvas:create .workdir.main
+  pack .workdir.main.pw -side bottom -fill both -expand yes
 
   set cvsglb(current_selection) {}
 
@@ -1114,12 +1110,17 @@ proc setup_dir { } {
     close $fileId
   } else {
     if {$insvn} {
-      set cmd [exec::new "svn propget svn:ignore"]
-      set contents [split [$cmd\::output] "\n"]
-      foreach line $contents {
-        append cvscfg(ignore_file_filter) " $line"
+      # Have to do eval exec because we need the error output
+      set command "svn propget svn:ignore"
+      gen_log:log C "$command"
+      set ret [catch {eval "exec $command"} output]
+      if {$ret} {
+        cvsfail $output
+        return
+      }
+      foreach infoline [split $output "\n"] {
+        append cvscfg(ignore_file_filter) " $infoline"
       }
-      $cmd\::destroy
     }
   }
 
@@ -1317,21 +1318,6 @@ proc are_you_sure {mess args} {
   return 1
 }
 
-proc busy_start {w} {
-
-  foreach widget [winfo children $w] {
-    catch {$widget config -cursor watch}
-  }
-  update idletasks
-}
-
-proc busy_done {w} {
-
-  foreach widget [winfo children $w] {
-    catch {$widget config -cursor ""}
-  }
-}
-
 proc workdir_print_file {args} {
   global cvscfg
 
@@ -1368,11 +1354,9 @@ proc cvsroot_check { dir } {
   foreach {incvs insvn inrcs} {0 0 0} {break}
 
   if {[file isfile [file join $dir CVS Root]]} {
-    set incvs 1
-    read_cvs_dir [file join $dir CVS]
+    set incvs [ read_cvs_dir [file join $dir CVS] ]
   } elseif {[file isfile [file join $dir .svn entries]]} {
-    set insvn 1
-    read_svn_dir $dir
+    set insvn [ read_svn_dir $dir ]
   } else {
     set rcsdir [file join $dir RCS]
     if {[file exists $rcsdir]} {
@@ -1406,12 +1390,6 @@ proc cvsroot_check { dir } {
   return [list $incvs $insvn $inrcs]
 }
 
-proc nop {} {}
-
-proc disabled {} {
-  cvsok "Command disabled." .workdir
-}
-
 proc isCmDirectory { file } {
   #gen_log:log T "ENTER ($file)"
   switch -- $file  {
